﻿##############################################################################
##
## Power Shell Module for managing Proxy Gateway Server
##
## Utilities to work with Gateway Administration Object
##
##############################################################################

# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Constants
Import-Module Proxy-Gateway-ADs

# Test if Gateway Administration Control Object has the right properties and events
# Otherwise its an instance of a COM object and not of .NET interop assembly
function Test-GatewayObject($prxGWS)
{
    if( 
        -not $prxGWS `
        -or `
        -not ($prxGWS.GetType().FullName -eq 'ProxyGWLib.ProxyGWClass') `
        -or `
        -not ($prxGWS.currentStation))
    {
        Write-Error -message "Error: Gateway SDK Object is not an instance of a COM object and not of .NET interop assembly." 
        return $false
    }

    return $true
}

# Finds a Gateway's group with given name and returns it in the ProxyGWGroups Collection.
function Find-Group($proxyGWGroups, [string]$name)
{
    foreach($group in $proxyGWGroups)
    {
        if($group.groupName -eq $name)
        {
            return $group
        }
    }
    
    return $null
}

# Copy Host to Groups with the same names as Organizational Units to wich Host belongs according to default AD
function Copy-HostToDesignatedOUs($prxGWS, $hostObject)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    # Get Host's OUs
    $adObj = Get-ComputerInDefaultAD $hostObject.machine
    if(-not $adObj)
    {
        Write-Warning -Message "Host machine: $hostObject.machine, wasn't found in default AD."
        return
    }

    # Test if Host is in Group with the same name as OU
    $OUs = ([adsi]($adObj.GetDirectoryEntry().parent)).ou
    foreach($ou in $OUs)
    {
        # Find Gateway Group with the same name
        $group = $prxGWS.getGroups() | ? { $_.groupName -eq [string]$ou }
        if(-not $group)
        {
            Write-Warning -Message "Group: $ou, wasn't found on the Gateway."
            continue
        }

        # Check if Host is in the Group        
        if(-not ($group.getHosts() | ? { $_.machine -eq $hostObject.machine }))
        {
            # Add Host to Group
            $group.addHost($hostObject)
            Write-Host "Host: <$($hostObject.machine)> added to <$($group.groupName)> group."
        }
    }
}

function Copy-HostToGroups($prxGWS, $host_object, $group_names_list, $forceGroupCreation)
{
    if(-not ([bool]$group_names_list))
    {
        # Empty Group list - do nothing
        return
    }
    
    # First make sure the Host is managed
    if([int]$host_object.isManaged -eq 0)
    {
        $host_object.isManaged = -1
    }
    
    # Go through Groups in the list
    foreach($group_name in $group_names_list)
    {
        # Find Group with this name
        $group = Find-Group -proxyGWGroups $prxGWS.getGroups() -name $group_name
        if((-not $group) -and $forceGroupCreation)
        {
            # If Group with this name is not found and $forceGroupCreation == $true - create group
            $prxGWS.addGroup($group_name, "$group_name Group")
            $group = Find-Group -proxyGWGroups $prxGWS.getGroups() -name $group_name
        }
        
        if($group)
        {
            # Check if Host is already in the Group
            if($group.getworkstations() | where { $_.key -eq $host_object.key })
            {
                continue
            }
            
            # Copy Host to target group
            $group.addHost($host_object)
            Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
        }
    }
}

function Remove-HostFromGroups($prxGWS, $host_object, $group_names_list)
{
    if(-not ([bool]$group_names_list))
    {
        # Empty Group list - do nothing
        return
    }
    
    # First make sure the Host is managed
    if([int]$host_object.isManaged -eq 0)
    {
        # Unmanaged Host - do nothing
        return
    }
    
    # Go through Groups in the list
    foreach($group_name in $group_names_list)
    {
        # Find Group with this name
        $group = Find-Group -proxyGWGroups $prxGWS.getGroups() -name $group_name
        if($group)
        {
            # Check if Host is in the Group
            if($group.getworkstations() | where { $_.key -eq $host_object.key })
            {
                $group.removeHost($host_object)
                Write-Host "Host: <$($host_object.machine)> removed from <$group_name> group."
            }
        }
    }
}

function Sync-HostInGroups($prxGWS, $host_object, $group_names_list, $forceGroupCreation)
{
    $groups_intersection_list_and_gateway = @()
    
    #
    # Iterate through all Gateway's Groups
    #
    foreach($group in $prxGWS.getGroups())
    {
        # Look only at Normal Groups (not System, All Hosts, etc.)
        if($group.GroupType -ne $groupTypeNormal)
        {
            continue
        }
        
        $group_name = $group.groupName
        
        if($group_names_list -contains $group_name)
        {
            # If Group is in the list
            $groups_intersection_list_and_gateway += $group_name
            
            # Check if Host is not in the Group
            if(-not ($group.getworkstations() | where { $_.key -eq $host_object.key }))
            {
                # Copy Host to the Group
                $group.addHost($host_object)
                Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
            }
        }
        else
        {
            # If Group is not in the list 
            
            # Check if Host is in the Group
            if($group.getworkstations() | where { $_.key -eq $host_object.key })
            {
                # Remove Host from the Group
                $group.removeHost($host_object)
                Write-Host "Host: <$($host_object.machine)> removed from <$group_name> group."
            }            
        }
    }
    
    #
    # Iterate through all groups which are in the list but not in the Gateway
    #
    if(-not $forceGroupCreation)
    {
        return
    }
    
    foreach($group_name in $group_names_list)
    {
        if($groups_intersection_list_and_gateway -contains $group_name)
        {
            continue
        }
        
        # Create group
        $prxGWS.addGroup($group_name, "$group_name Group")
        
        $group = Find-Group -proxyGWGroups $prxGWS.getGroups() -name $group_name
        if($group)
        {
            # Copy Host to target group
            $group.addHost($host_object)
            Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
        }
    }
}

# Iterates through all Managed and Unmanaged Hosts on the Gateway and calls $policy for each Host with Host Object as a parameter. 
# Next, it gets list of Group's Names returned from the $policy and makes sure this Host belongs to all those Groups, making it 
# Managed first if list is not empty.
function Copy-HostToGroupsByPolicy($prxGWS, $policy, $forceGroupCreation=$false)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    $prxGWS.getWorkstations() `
    | where { $_.machine -ne 'virtual' } `
    | foreach { Copy-HostToGroups -prxGWS $prxGWS -host_object $_ -group_names_list (& $policy $_) -forceGroupCreation $forceGroupCreation }
}

# Removes Host from groups by given policy (works the same as Copy-HostToGroupsByPolicy but removing instead of copying).
function Remove-HostFromGroupsByPolicy($prxGWS, $policy)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    $prxGWS.getWorkstations() `
    | where { ($_.machine -ne 'virtual') -and ($_.isManaged -eq -1) } `
    | foreach { Remove-HostFromGroups -prxGWS $prxGWS -host_object $_ -group_names_list (& $policy $_) }
}

# Iterates through all Managed and Unmanaged Hosts on the Gateway and calls $policy for each Host with Host Object as a parameter. 
# Next, it gets list of Group's Names returned from the $policy and makes sure this Host belongs to all those Groups which are 
# in the list, and doesn't belongs to the groups which are not in the list but exists on the Gateway.
function Sync-HostInGroupsByPolicy($prxGWS, $policy, $forceGroupCreation=$false)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    $prxGWS.getWorkstations() `
    | where { $_.machine -ne 'virtual' } `
    | foreach { Sync-HostInGroups -prxGWS $prxGWS -host_object $_ -group_names_list (& $policy $_) -forceGroupCreation $forceGroupCreation }
}

#
# Module exports
#
Export-ModuleMember -Function Test-GatewayObject, Find-*, Copy-*, Remove-*, Sync-*

# SIG # Begin signature block
# MIIXcgYJKoZIhvcNAQcCoIIXYzCCF18CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU3+XYqNmlzQajNHQfYkvPukMP
# BMugghLuMIIDnzCCAoegAwIBAgIQeaKlhfnRFUIT2bg+9raN7TANBgkqhkiG9w0B
# AQUFADBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xKzAp
# BgNVBAMTIlZlcmlTaWduIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EwHhcNMTIw
# NTAxMDAwMDAwWhcNMTIxMjMxMjM1OTU5WjBiMQswCQYDVQQGEwJVUzEdMBsGA1UE
# ChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xNDAyBgNVBAMTK1N5bWFudGVjIFRpbWUg
# U3RhbXBpbmcgU2VydmljZXMgU2lnbmVyIC0gRzMwgZ8wDQYJKoZIhvcNAQEBBQAD
# gY0AMIGJAoGBAKlZZnTaPYp9etj89YBEe/5HahRVTlBHC+zT7c72OPdPabmx8LZ4
# ggqMdhZn4gKttw2livYD/GbT/AgtzLVzWXuJ3DNuZlpeUje0YtGSWTUUi0WsWbJN
# JKKYlGhCcp86aOJri54iLfSYTprGr7PkoKs8KL8j4ddypPIQU2eud69RAgMBAAGj
# geMwgeAwDAYDVR0TAQH/BAIwADAzBgNVHR8ELDAqMCigJqAkhiJodHRwOi8vY3Js
# LnZlcmlzaWduLmNvbS90c3MtY2EuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMI
# MDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNp
# Z24uY29tMA4GA1UdDwEB/wQEAwIHgDAeBgNVHREEFzAVpBMwETEPMA0GA1UEAxMG
# VFNBMS0zMB0GA1UdDgQWBBS0t/GJSSZg52Xqc67c0zjNv1eSbzANBgkqhkiG9w0B
# AQUFAAOCAQEAHpiqJ7d4tQi1yXJtt9/ADpimNcSIydL2bfFLGvvV+S2ZAJ7R55uL
# 4T+9OYAMZs0HvFyYVKaUuhDRTour9W9lzGcJooB8UugOA9ZresYFGOzIrEJ8Byyn
# PQhm3ADt/ZQdc/JymJOxEdaP747qrPSWUQzQjd8xUk9er32nSnXmTs4rnykr589d
# nwN+bid7I61iKWavkugszr2cf9zNFzxDwgk/dUXHnuTXYH+XxuSqx2n1/M10rCyw
# SMFQTnBWHrU1046+se2svf4M7IV91buFZkQZXZ+T64K6Y57TfGH/yBvZI1h/MKNm
# oTkmXpLDPMs3Mvr1o43c1bCj6SU2VdeB+jCCA8QwggMtoAMCAQICEEe/GZXfjVJG
# Q/fbbUgNMaQwDQYJKoZIhvcNAQEFBQAwgYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQI
# EwxXZXN0ZXJuIENhcGUxFDASBgNVBAcTC0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZU
# aGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBDZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZU
# aGF3dGUgVGltZXN0YW1waW5nIENBMB4XDTAzMTIwNDAwMDAwMFoXDTEzMTIwMzIz
# NTk1OVowUzELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSsw
# KQYDVQQDEyJWZXJpU2lnbiBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBMIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqcqypMzNIK8KfYmsh3XwtE7x38EP
# v2dhvaNkHNq7+cozq4QwiVh+jNtr3TaeD7/R7Hjyd6Z+bzy/k68Numj0bJTKvVIt
# q0g99bbVXV8bAp/6L2sepPejmqYayALhf0xS4w5g7EAcfrkN3j/HtN+HvV96ajEu
# A5mBE6hHIM4xcw1XLc14NDOVEpkSud5oL6rm48KKjCrDiyGHZr2DWFdvdb88qiaH
# XcoQFTyfhOpUwQpuxP7FSt25BxGXInzbPifRHnjsnzHJ8eYiGdvEs0dDmhpfoB6Q
# 5F717nzxfatiAY/1TQve0CJWqJXNroh2ru66DfPkTdmg+2igrhQ7s4fBuwIDAQAB
# o4HbMIHYMDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3Au
# dmVyaXNpZ24uY29tMBIGA1UdEwEB/wQIMAYBAf8CAQAwQQYDVR0fBDowODA2oDSg
# MoYwaHR0cDovL2NybC52ZXJpc2lnbi5jb20vVGhhd3RlVGltZXN0YW1waW5nQ0Eu
# Y3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIBBjAkBgNVHREE
# HTAbpBkwFzEVMBMGA1UEAxMMVFNBMjA0OC0xLTUzMA0GCSqGSIb3DQEBBQUAA4GB
# AEpr+epYwkQcMYl5mSuWv4KsAdYcTM2wilhu3wgpo17IypMT5wRSDe9HJy8AOLDk
# yZNOmtQiYhX3PzchT3AxgPGLOIez6OiXAP7PVZZOJNKpJ056rrdhQfMqzufJ2V7d
# uyuFPrWdtdnhV/++tMV+9c8MnvCX/ivTO1IbGzgn9z9KMIIFcTCCBFmgAwIBAgIQ
# XM5xW+LS1ji4A9NViFFAMDANBgkqhkiG9w0BAQUFADCBtDELMAkGA1UEBhMCVVMx
# FzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVz
# dCBOZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cu
# dmVyaXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3Mg
# MyBDb2RlIFNpZ25pbmcgMjAxMCBDQTAeFw0xMjA2MTUwMDAwMDBaFw0xNDA3MDky
# MzU5NTlaMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UECBMNTWFzc2FjaHVzZXR0czEP
# MA0GA1UEBxMGQm9zdG9uMR0wGwYDVQQKFBRQcm94eSBOZXR3b3JrcywgSW5jLjE+
# MDwGA1UECxM1RGlnaXRhbCBJRCBDbGFzcyAzIC0gTWljcm9zb2Z0IFNvZnR3YXJl
# IFZhbGlkYXRpb24gdjIxHTAbBgNVBAMUFFByb3h5IE5ldHdvcmtzLCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuORukVR3ArkRbmIr7uMEWljY
# GxNuA2MUK0RKXLO8sswGaEPlSq2KeysSQZVScPGcJtkNj+lIsZ5ECM+bYdAJ4Svv
# BTEqt0HKHsovnv7UKFUCoDOHvVZThWA7+58iFrbMMa/QrPWaqBqYMBGyCtRRZjwf
# pE5kWKe617Kcz0w+myqJv/FmOg7mXe5/j25U21/C48q6uE+CzTvm/fv/DhCOZrMH
# ts641LnzX5I3SNv24pMZ0eUU7hGG9J42u+m3D/yC98DkzWIfYnNCcM2COQCkx2FR
# 6MoB2ny6PN+YpdgF2nD71t2LuJMDxVF6PGbE9b+aNMfraRq1rwPfT8BzPwmavQID
# AQABo4IBezCCAXcwCQYDVR0TBAIwADAOBgNVHQ8BAf8EBAMCB4AwQAYDVR0fBDkw
# NzA1oDOgMYYvaHR0cDovL2NzYzMtMjAxMC1jcmwudmVyaXNpZ24uY29tL0NTQzMt
# MjAxMC5jcmwwRAYDVR0gBD0wOzA5BgtghkgBhvhFAQcXAzAqMCgGCCsGAQUFBwIB
# FhxodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMHEGCCsGAQUFBwEBBGUwYzAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVy
# aXNpZ24uY29tMDsGCCsGAQUFBzAChi9odHRwOi8vY3NjMy0yMDEwLWFpYS52ZXJp
# c2lnbi5jb20vQ1NDMy0yMDEwLmNlcjAfBgNVHSMEGDAWgBTPmanqeyb0S8mOj9fw
# BSbv49KnnTARBglghkgBhvhCAQEEBAMCBBAwFgYKKwYBBAGCNwIBGwQIMAYBAQAB
# Af8wDQYJKoZIhvcNAQEFBQADggEBABDAtzEM2dWEt6wcvi3fSe9Y+hwIS0Oi6Rmf
# QkWMWwHKXvZCOOzuq2i2ywCXnyPz+Fb7waMen0GK2DkfNOwjbjO3yKfDpzv/MHTY
# 1pQjbyf2ySqrRJZ1iqaTDdTe/dhz2PEnp3wIMCM3wX5OCfHEKDUx4BYcLwxaEMAY
# V0j/wJ/vIBdpbDG3GfBUpeRavYjYAEIau7HCMN7lKvWODP09JaePLwfqNzDKB+GY
# cjiEI0gZMY2U4gJ5VygYHGRQslgZ+gVqGg9A6wqWSjqLXBy8C5CRNQmds+e6081Y
# b3WvPbwtI1lGTNmJDCTkJuXDk2S5LDkOEg4DpilD1nN7MiqzRf0wggYKMIIE8qAD
# AgECAhBSAOWqJVb8GobtlsnUSzPHMA0GCSqGSIb3DQEBBQUAMIHKMQswCQYDVQQG
# EwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWdu
# IFRydXN0IE5ldHdvcmsxOjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMu
# IC0gRm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENs
# YXNzIDMgUHVibGljIFByaW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# NTAeFw0xMDAyMDgwMDAwMDBaFw0yMDAyMDcyMzU5NTlaMIG0MQswCQYDVQQGEwJV
# UzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRy
# dXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3
# dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFz
# cyAzIENvZGUgU2lnbmluZyAyMDEwIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA9SNLXqXXirsy6dRX9+/kxyZ+rRmY/qidfZT2NmsQ13WBMH8EaH/L
# K3UezR0IjN9plKc3o5x7gOCZ4e43TV/OOxTuhtTQ9Sc1vCULOKeMY50Xowilq7D7
# zWpigkzVIdob2fHjhDuKKk+FW5ABT8mndhB/JwN8vq5+fcHd+QW8G0icaefApDw8
# QQA+35blxeSUcdZVAccAJkpAPLWhJqkMp22AjpAle8+/PxzrL5b65Yd3xrVWsno7
# VDBTG99iNP8e0fRakyiF5UwXTn5b/aSTmX/fze+kde/vFfZH5/gZctguNBqmtKdM
# fr27Tww9V/Ew1qY2jtaAdtcZLqXNfjQtiQIDAQABo4IB/jCCAfowEgYDVR0TAQH/
# BAgwBgEB/wIBADBwBgNVHSAEaTBnMGUGC2CGSAGG+EUBBxcDMFYwKAYIKwYBBQUH
# AgEWHGh0dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9jcHMwKgYIKwYBBQUHAgIwHhoc
# aHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYTAOBgNVHQ8BAf8EBAMCAQYwbQYI
# KwYBBQUHAQwEYTBfoV2gWzBZMFcwVRYJaW1hZ2UvZ2lmMCEwHzAHBgUrDgMCGgQU
# j+XTGoasjY5rw8+AatRIGCx7GS4wJRYjaHR0cDovL2xvZ28udmVyaXNpZ24uY29t
# L3ZzbG9nby5naWYwNAYDVR0fBC0wKzApoCegJYYjaHR0cDovL2NybC52ZXJpc2ln
# bi5jb20vcGNhMy1nNS5jcmwwNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhho
# dHRwOi8vb2NzcC52ZXJpc2lnbi5jb20wHQYDVR0lBBYwFAYIKwYBBQUHAwIGCCsG
# AQUFBwMDMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBWZXJpU2lnbk1QS0ktMi04
# MB0GA1UdDgQWBBTPmanqeyb0S8mOj9fwBSbv49KnnTAfBgNVHSMEGDAWgBR/02Wn
# wt3su/AwCfNDOfoCrzMxMzANBgkqhkiG9w0BAQUFAAOCAQEAViLmNKTEYctIuQGt
# VqhkD9mMkcS7zAzlrXqgIn/fRzhKLWzRf3EafOxwqbHwT+QPDFP6FV7+dJhJJIWB
# JhyRFEewTGOMu6E01MZF6A2FJnMD0KmMZG3ccZLmRQVgFVlROfxYFGv+1KTteWsI
# DEFy5zciBgm+I+k/RJoe6WGdzLGQXPw90o2sQj1lNtS0PUAoj5sQzyMmzEsgy5Af
# XYxMNMo82OU31m+lIL006ybZrg3nxZr3obQhkTNvhuhYuyV8dA5Y/nUbYz/OMXyb
# jxuWnsVTdoRbnK2R+qztk7pdyCFTwoJTY68SDVCHERs9VFKWiiycPZIaCJoFLseT
# pUiR0zGCA+4wggPqAgEBMIHJMIG0MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVy
# aVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5
# BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20v
# cnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmlu
# ZyAyMDEwIENBAhBcznFb4tLWOLgD01WIUUAwMAkGBSsOAwIaBQCgeDAYBgorBgEE
# AYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwG
# CisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBTo6jyy
# QK9MIeydeFXEm24P+q8G7jANBgkqhkiG9w0BAQEFAASCAQAHX4GFY5fQw+nQoqlP
# 7uDp8B7ypuhRo14OKvKiZFe3FvZt54Rq5B5t34/OVwfGSsePlNTvGmpqVilgmt+3
# RodQRVc/DkuIo55SIl0MM5vPseRzrIp+a1kKlv1xDvZa4/eZ4/uTEcHmjrmjaPdC
# ejoDxEM+ll3si9T3FthtZJIxkCCWg5HR6OVibAqFQCLre2m2iKs3riQnktgwmeMU
# WU8/QpDOLfNVyw2y42+mduhWRZBw67nXdN94hqQqwa8RzCtP8wbU83BX8k9xSn8T
# A5kkLdqpmypPPOUInk407sOZiW+yO6uppmB9eJtXyVWGmTQieOjDSM9TDhj2qZUs
# XqrvoYIBfzCCAXsGCSqGSIb3DQEJBjGCAWwwggFoAgEBMGcwUzELMAkGA1UEBhMC
# VVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSswKQYDVQQDEyJWZXJpU2lnbiBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBAhB5oqWF+dEVQhPZuD72to3tMAkGBSsO
# AwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEP
# Fw0xMjA4MDgyMjU1NTVaMCMGCSqGSIb3DQEJBDEWBBQSyaa0yC3vrBsD2qzkDCuy
# z5SuyDANBgkqhkiG9w0BAQEFAASBgH9ZNkeRr8HntKWs4DyrOuawQbreyDrsE08L
# wOe/SCmtARgfG9e6HzLzY9Pu/sN/FPYy/urGdNY4ypQVMawuL69tHpPSQzaogALT
# XQNx6G4fB0DbaoFAViAHXxq96YsXkiUxMYwpTOM5Kbafv7R8CSBwDXWJPakYEuOU
# VMKy7CfY
# SIG # End signature block
