﻿##############################################################################
##
## Power Shell Module for managing Proxy Gateway Server
##
## Exported functions:
##  Connect-Gateway
##  Get-Connection
##  Get-GatewayObject
##  Disconnect-Gateway
##  Test-Connection
##  Register-UnmanagedHostChange
##  Register-GroupMembershipChange
##
## Exported examples:
##  $script:example_event_unmanaged
##  $script:example_event_group_membership
##
##
##
##############################################################################

# There can be only one active Gateway connection in one instance of the PS script. "activeConnection" holds it.
$script:activeConnection = $null

# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Gateway-Utils
Import-Module Proxy-Gateway-ADs
Import-Module Proxy-ConnectionEvents

# Check if we are running in "STA" apartment. In this case the window's message notification between MTA and STA parts of ProxyGW object
# is not working unless message pump gets called explicitly.
if($Host.Runspace.ApartmentState -eq "STA") 
{
    Write-Warning "WARNING: You are running in STA Apartment mode.
This occurs in the PowerShell ISE, or by default in PowerShell v3 and later.
The Gateway Client's COM object will not function properly when running in
the STA. Specifically, if the network connection drops or fails, that will
not be reflected in the Gateway object state."
}


<#

.FUNCTION NAME

Test-Connection

.SYNOPSIS

Tests if there is current established connection to the Gateway Server exists. 

.RELATED
Connect-Gateway
Disconnect-Gateway
Get-Connection

.EXAMPLE

PS >if(Test-Connection) { Disconnect-Gateway }

#>
function Test-Connection
{
    if($script:activeConnection -and $script:activeConnection.gateway -and $script:activeConnection.gateway.currentStation)
    {
        return $true
    }

    return $false
}

<#

.FUNCTION NAME

Connect-Gateway

.SYNOPSIS

This method establishes new connection to Proxy Gateway Server. In the case of success it returns 
$true and if connection fails - $false. When user finish to work with the server, Disconnect-Gateway should
be called.

There can be only one active connection to Gateway Server and Connect-Gateway will return $true and 
write an error in case such connection already exists. Actions performed by functions in this module, 
performed on the "active connection" only. 

.RELATED
Disconnect-Gateway
Test-Connection
Get-Connection

.EXAMPLE

PS >$connected = Connect-Gateway -station "hostname.domain.com" -protocol "TCP" -credentials "domain\user|password"
PS >
PS >$connected = Connect-Gateway -station "hostname.domain.com"
PS >Connect-Gateway "hostname.domain.com"
PS >Connect-Gateway "hostname.domain.com" -protocol "IP"
PS >Connect-Gateway "hostname.domain.com" -credential "boston\ivan_k|pass"
PS >
PS >if($connected) { Disconnect-Gateway }
PS >
PS >
PS >

#>
function Connect-Gateway([string]$station, [string]$protocol='TCP', [string]$credentials='')
{
    if(Test-Connection)
    {
        Write-Error -message "Error: Already connected to the Gateway: $($script:activeConnection.gateway.currentStation.displayName)" `
            -Category ResourceExists -ErrorId $_ERROR_ID_AlreadyConnected
        
        return $true
    }

    # Create a new Proxy SDK object of Proxy.ProxyGW type
    $prxGW = New-Object -COM Proxy.ProxyGW
    if( 
        -not $prxGW `
        -or `
        -not ($prxGW.GetType().FullName -eq 'ProxyGWLib.ProxyGWClass'))
    { 
        Write-Error -message "Error: Failed to create Proxy.ProxyGW COM object." `
            -Category ObjectNotFound -ErrorId $_ERROR_ID_COMError
        
        return $false 
    }
    
    # Suppress UI messages from control
    $prxGW.displayMessages = $false
    
    # Register actions for connection events. The actions are; - issuing of a new
    # event with custom source "Custom.ProxyGW.ConnectionEvent" and an argument 
    # indicating actual ProxyGW event, in order to wait for one event in the script code.
    Register-ObjectEvent $prxGW onConnectSucceeded -SourceIdentifier onConnectSucceeded -MessageData $_EVENT_ConnectSucceeded
    Register-ObjectEvent $prxGW onConnectFailed -SourceIdentifier onConnectFailed -MessageData $_EVENT_ConnectFailed
    Register-ObjectEvent $prxGW onDisconnect -SourceIdentifier onDisconnect -MessageData $_EVENT_Disconnect
    
    Write-Verbose "Connecting to:$protocol, $station, $credentials"
    
    # Initiate connection to the Proxy Gateway with the given parameters
    $prxGW.connectToGatewayByName($protocol, $station, $credentials)

    # Wait for connection to complete with timeout 30sec
    $waitRes = Wait-EventWithMessageLoop "on*" -Timeout 30
    if($waitRes)
    {
        # Get the completion event
        $gswEvent = Get-Event "on*" -ErrorAction:Continue
		if($gswEvent)
		{
			Write-Verbose "Got event $($gswEvent.SourceIdentifier)"
	        if($gswEvent.MessageData -eq $_EVENT_ConnectSucceeded)
	        {
	            Write-Verbose "Connection to:$station - is successful."
	            
	            # Clean completion events from the queue
	            Remove-Event "on*"

	            # Remove event handlers
	            ("onConnectSucceeded", "onConnectFailed", "onDisconnect") | % { Unregister-Event -SourceIdentifier $_ -Force }
	            
	            # Remember new connection as an active connection
	            $script:activeConnection = New-Object PsObject -Property @{
	                    gateway = $prxGW
	                }
	                
	            Write-Verbose "Connected."
	            
	            return $true
	        }
			else
			{
		        Write-Error -message "Error: Failed to connect to the Gateway: $station, Error: $($gswEvent.SourceArgs)" `
		            -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
			}
		}
		else
		{
	        Write-Error -message "Error: Failed to connect to the Gateway: $station, Error: Unknown" `
	            -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
		}
    }
    else
    {
        Write-Error -message "Connection to the Gateway timed out." `
            -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
    }

    # Clean events from the queue
    Remove-Event "on*" -ErrorAction:SilentlyContinue
	
    # Remove event handlers
    ("onConnectSucceeded", "onConnectFailed", "onDisconnect") | % { Unregister-Event -SourceIdentifier $_ -Force }
    
    [System.Runtime.Interopservices.Marshal]::ReleaseComObject($prxGW) | Out-Null
    $prxGW = $null
    
    return $false
}

function Clean-State
{
    if($script:activeConnection)
    {
        if($script:activeConnection.events) {
            $script:activeConnection.events.Values | % { Unregister-Event -SourceIdentifier $_.source -Force }
            $script:activeConnection.events.Values | % { Remove-Job $_.event -Force }
            $script:activeConnection.events = $null
        }
        
        if($script:activeConnection.gateway) {
            [System.Runtime.Interopservices.Marshal]::ReleaseComObject($script:activeConnection.gateway) | Out-Null
            $script:activeConnection.gateway = $null
        }
    }
    
    $script:activeConnection = $null
    
    Remove-Event "on*" -ErrorAction:SilentlyContinue
}

<#

.FUNCTION NAME

Disconnect-Gateway

.SYNOPSIS

This method disconnects from a Gateway to which a connection was previously established. 

.RELATED
Connect-Gateway
Test-Connection
Get-Connection

.EXAMPLE

PS >if(Test-Connection) { Disconnect-Gateway }

#>
function Disconnect-Gateway
{
    if(-not (Test-Connection))
    {
        Clean-State
        Write-Verbose "Not connected."
        return
    }

    Register-ObjectEvent $script:activeConnection.gateway onDisconnect -SourceIdentifier onDisconnect
	
    $script:activeConnection.gateway.disconnect()
    
    $null = Wait-EventWithMessageLoop "onDisconnect" -Timeout 30

	# Clean completion events from the queue
    Remove-Event "onDisconnect" 

    # Remove event handler
	Unregister-Event -SourceIdentifier "onDisconnect" -Force
	
    Clean-State
    
    Write-Verbose "Disconnected."
}

<#

.FUNCTION NAME

Get-Connection

.SYNOPSIS

Returns current connection to Gateway Server or $null if there is no one established. 

.RELATED
Connect-Gateway
Test-Connection
Disconnect-Gateway
Get-GatewayObject

.EXAMPLE

PS >if(Test-Connection) { Write-Host (Get-Connection).gateway.currentStation }

#>
function Get-Connection
{
    if(Test-Connection)
    {
        return $script:activeConnection
    }
    
    return $null
}

<#

.FUNCTION NAME

Get-GatewayObject

.SYNOPSIS

Returns current Gateway Server Object or $null if there is no connection to Gateway Server were established. 

.RELATED
Connect-Gateway
Test-Connection
Disconnect-Gateway
Get-Connection

.EXAMPLE

PS >if(Test-Connection) { Write-Host (Get-GatewayObject).currentStation }

#>
function Get-GatewayObject
{
    if(Test-Connection)
    {
        return $script:activeConnection.gateway
    }
    
    return $null
}

function Register-GroupMembershipChange([System.Management.Automation.ScriptBlock]$action)
{
    if(-not (Test-Connection))
    {
        Write-Error "Error: Not connected to Proxy Gateway." -ErrorId $_ERROR_ID_NotConnected
        return $false
    }

    # Enable group membership notifications on the Gateway Server
    (Get-Connection).gateway.enableChangeNotifications([ProxyGWLib.GatewayCollection]::collectGroupMemberships, -1, "", -1)
    
    # Enable group membership event's handling in script
    $eventSource = [System.Guid]::NewGuid().Guid
    $newEvent = Register-ObjectEvent $script:activeConnection.gateway onGroupMembershipChanged `
        -SourceIdentifier $eventSource -MessageData ($script:activeConnection.gateway) -Action $action
        
    if(-not $script:activeConnection.events)
    {
        Add-Member -InputObject $script:activeConnection NoteProperty events @{}
    }
    $script:activeConnection.events.GroupMembershipChange = (New-Object PsObject -Property @{source = $eventSource; event = $newEvent})
    
    return $true 
}

function Register-UnmanagedHostChange([System.Management.Automation.ScriptBlock]$action)
{
    if(-not (Test-Connection))
    {
        Write-Error "Error: Not connected to Proxy Gateway." -ErrorId $_ERROR_ID_NotConnected
        return $false
    }

    # Enable group membership notifications on the Gateway Server
    (Get-Connection).gateway.enableChangeNotifications([ProxyGWLib.GatewayCollection]::collectUnmanagedHosts, -1, "", -1)
    
    # Enable group membership event's handling in script
    $eventSource = [System.Guid]::NewGuid().Guid
    $newEvent = Register-ObjectEvent $script:activeConnection.gateway onUnmanagedHostDataChanged `
        -SourceIdentifier $eventSource -MessageData ($script:activeConnection.gateway) -Action $action
        
    if(-not $script:activeConnection.events)
    {
        Add-Member -InputObject $script:activeConnection NoteProperty events @{}
    }
    $script:activeConnection.events.UnmanagedHostDataChanged = (New-Object PsObject -Property @{source = $eventSource; event = $newEvent})
    
    return $true 
}

function Register-WorkstationsDataChange([System.Management.Automation.ScriptBlock]$action)
{
    if(-not (Test-Connection))
    {
        Write-Error "Error: Not connected to Proxy Gateway." -ErrorId $_ERROR_ID_NotConnected
        return $false
    }

    # Enable Workstations Data Change notifications on the Gateway Server
    (Get-Connection).gateway.enableChangeNotifications([ProxyGWLib.GatewayCollection]::collectWorkstations, -1, "", -1)
    
    # Enable "Property of a workstation-type Host has changed" event's handling in script
    $eventSource = [System.Guid]::NewGuid().Guid
    $newEvent = Register-ObjectEvent $script:activeConnection.gateway onWorkstationDataChanged `
        -SourceIdentifier $eventSource -MessageData ($script:activeConnection.gateway) -Action $action
        
    if(-not $script:activeConnection.events)
    {
        Add-Member -InputObject $script:activeConnection NoteProperty events @{}
    }
    $script:activeConnection.events.WorkstationsDataChange = (New-Object PsObject -Property @{source = $eventSource; event = $newEvent})
    
    return $true 
}

<#
    Example of Gateway Administration Object event handling for changes in collection of Unmanaged Hosts.
    
    Usage: Register-UnmanagedHostChange $example_event_unmanaged_action
#>
$script:example_event_unmanaged_action = {
	# Import Proxy common PS utilities
	Import-Module Proxy-Utils
    # Import WMI helpers for AD 
    Import-Module Proxy-Gateway-ADs
    # Import Gateway helpers
    Import-Module Proxy-Gateway-Utils
    
    # Name of the Group where Host will be moved automatically in the case 
    # it belongs to default AD
    $targetGroup = 'autoPS'
    $targetGroupDescription = 'PowerShell example group'
    
    # Gateway Administration Object passed as event MessageData from the script
	$gateway	= $event.MessageData
    # Event identification from Gateway Administration Object
	$eventID	= $event.SourceArgs[0]
    # Host object
	$eventHost	= $event.SourceArgs[1]
    
	Write-Host "
        Proxy 'example_event_unmanaged'
		Connected to:  $(($gateway).currentStation.displayName)
		Event ID:      $($eventID)
		Host Name:     $($eventHost.machine)
	"
    
    if($eventID -eq [ProxyGWLib.HostProperty]::hostpropAdded)
    {
        Write-Host "New Host is added to 'Unmanaged' group: <$($eventHost.machine)>."
        
        if(Find-ComputerInDomain $eventHost.machine)
        {
            Write-Host "New Unmanaged Host: <$($eventHost.machine)> is in AD, moving it to group <$targetGroup>."
            
            $to = Find-Group -proxyGWGroups $gateway.getGroups() -name $targetGroup
            if(-not $to)
            {
                $gateway.addGroup($targetGroup, $targetGroupDescription)
                $to = Find-Group -proxyGWGroups $gateway.getGroups() -name $targetGroup
            }
            
            if($to)
            {
                Write-Host "Target Group <$targetGroup> found."
                
                # Move Host to Managed hosts list
                $eventHost.isManaged = -1

                Write-Host "Host: <$($eventHost.machine)> moved to Managed Hosts."
                
                # Move Host to target group
                $to.addHost($eventHost)
                
                Write-Host "Host: <$($eventHost.machine)> added to <$targetGroup> group."
            }
        }
        else
        {
            Write-Host "New Unmanaged Host: <$($eventHost.machine)> is not in AD."
        }
    }
}

<#
    Template action for changes in Group membership of Managed Hosts.
    
    Usage: Register-GroupMembershipChange $template_event_group_membership_action
#>
$script:template_event_group_membership_action = {
	# Import Proxy common PS utilities
	Import-Module Proxy-Utils
    # Import WMI helpers for AD 
    Import-Module Proxy-Gateway-ADs
    # Import Gateway helpers
    Import-Module Proxy-Gateway-Utils
    
    # Gateway Administration Object passed as event MessageData from the script
	$gateway	= $event.MessageData
    # Event identification from Gateway Administration Object
	$eventID	= $event.SourceArgs[0]
    # Group object
	$eventGroup	= $event.SourceArgs[1]
    # Host object
	$eventHost	= $event.SourceArgs[2]
    
	Write-Host "
        Proxy 'template_event_group_membership_action'
		Connected to:	$(($gateway).currentStation.displayName)
		Event ID:		$($eventID)
		Group:			$($eventGroup.groupName)
		Host:			$($eventHost.name)
	"
}

<#
    Template action for changes in collection of Unmanaged Hosts.
    
    Usage: Register-UnmanagedHostChange $template_event_unmanaged_action
#>
$script:template_event_unmanaged_action = {
	# Import Proxy common PS utilities
	Import-Module Proxy-Utils
    # Import WMI helpers for AD 
    Import-Module Proxy-Gateway-ADs
    # Import Gateway helpers
    Import-Module Proxy-Gateway-Utils
    
    # Gateway Administration Object passed as event MessageData from the script
	$gateway	= $event.MessageData
    # Event identification from Gateway Administration Object
	$eventID	= $event.SourceArgs[0]
    # Host object
	$eventHost	= $event.SourceArgs[1]
    
	Write-Host "
        Proxy 'template_event_unmanaged_action'
		Connected to:  $(($gateway).currentStation.displayName)
		Event ID:      $($eventID)
		Host Name:     $($eventHost.machine)
	"
}

<#
    Template action for Workstation Data Change event.
    
    Usage: Register-WorkstationsDataChange $template_event_workstation_data_action
#>
$script:template_event_workstation_data_action = {
	# Import Proxy common PS utilities
	Import-Module Proxy-Utils
    # Import WMI helpers for AD 
    Import-Module Proxy-Gateway-ADs
    # Import Gateway helpers
    Import-Module Proxy-Gateway-Utils
    
    # Gateway Administration Object passed as event MessageData from the script
	$gateway	= $event.MessageData
    # Event identification from Gateway Administration Object
	$eventID	= $event.SourceArgs[0]
    # Host object
	$eventHost	= $event.SourceArgs[1]
    
	Write-Host "
        Proxy 'template_event_workstation_data_action'
		Connected to:  $(($gateway).currentStation.displayName)
		Event ID:      $($eventID), $($eventID.GetType())
		Host Name:     $($eventHost.machine)
	"
}

<#
    Action for Workstation Data Change event to copy added workstations to Groups with 
    the same names as Organizational Units to wich Host belongs according to default AD.
    
    Usage: Register-WorkstationsDataChange $copy_new_workstation_to_OUs_action
#>
$script:copy_new_workstation_to_OUs_action = {
	# Import Proxy common PS utilities
	Import-Module Proxy-Utils
    # Import WMI helpers for AD 
    Import-Module Proxy-Gateway-ADs
    # Import Gateway helpers
    Import-Module Proxy-Gateway-Utils
    
    # Gateway Administration Object passed as event MessageData from the script
	$gateway	= $event.MessageData
    # Event identification from Gateway Administration Object
	$eventID	= $event.SourceArgs[0]
    # Host object
	$eventHost	= $event.SourceArgs[1]
    
    if($eventID -ne [ProxyGWLib.HostProperty]::hostpropAdded)
    {
        return
    }
    
	Write-Host "
        Proxy 'copy_new_workstation_to_OUs_action'
		Connected to:  $(($gateway).currentStation.displayName)
		New Host: $($eventHost.machine) added. Try to copy it to designated Organizational Units
	"

    $null = Copy-HostToDesignatedOUs -prxGWS $gateway -hostObject $eventHost       
}


#
# Module exports
#
Export-ModuleMember -Function Connect-Gateway, Disconnect-Gateway, Test-Connection, Get-Connection, Get-GatewayObject, Register-* `
    -Variable _ERROR_ID_*, _EVENT_*, example_*, template_*, copy_new_workstation_to_OUs_action

# SIG # Begin signature block
# MIIX4gYJKoZIhvcNAQcCoIIX0zCCF88CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUB8JhlArPU3wnKWFzUIN/PjNX
# GTWgghLFMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggTeMIIDxqADAgECAgIDATANBgkqhkiG9w0BAQUFADBjMQswCQYDVQQGEwJVUzEh
# MB8GA1UEChMYVGhlIEdvIERhZGR5IEdyb3VwLCBJbmMuMTEwLwYDVQQLEyhHbyBE
# YWRkeSBDbGFzcyAyIENlcnRpZmljYXRpb24gQXV0aG9yaXR5MB4XDTA2MTExNjAx
# NTQzN1oXDTI2MTExNjAxNTQzN1owgcoxCzAJBgNVBAYTAlVTMRAwDgYDVQQIEwdB
# cml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRowGAYDVQQKExFHb0RhZGR5LmNv
# bSwgSW5jLjEzMDEGA1UECxMqaHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNv
# bS9yZXBvc2l0b3J5MTAwLgYDVQQDEydHbyBEYWRkeSBTZWN1cmUgQ2VydGlmaWNh
# dGlvbiBBdXRob3JpdHkxETAPBgNVBAUTCDA3OTY5Mjg3MIIBIjANBgkqhkiG9w0B
# AQEFAAOCAQ8AMIIBCgKCAQEAxC3VFYycJkzsMjXrX7hZAVqmYYFZO3Bjq+PcPccq
# uMkz03nkOu08MCOEjrMwFLayh8M9lVQEnt+Z3QslHiHeZSl+NaipVOv29zI51CZV
# la3v+/5Yhtee9ACNjCoMvUIEzqc/BPbugPKq71KhaWbavhqtXdosZuoaa7vlGlFK
# AC9Ix5h12LkpyO74Zm0KnLPz/Hh8ovij8rXD87l6kcGn5iUunKjtEmVuavYSRFNw
# MJXDnCtYKz0IdEryvlGwv4fQTCdYa7U1xZ2vFzH4C4/urYE2BYkImM86ryWHwEnq
# p/1n90WOl8wUOeI2hbV+Gjf9FvZxEZp0MBb+E5SjP4QNTwIDAQABo4IBMjCCAS4w
# HQYDVR0OBBYEFP2sYTKTbEXW4u6FX5q653aZaMznMB8GA1UdIwQYMBaAFNLEsNKR
# 1EwRcbNhyz2h/t2oatTjMBIGA1UdEwEB/wQIMAYBAf8CAQAwMwYIKwYBBQUHAQEE
# JzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5nb2RhZGR5LmNvbTBGBgNVHR8E
# PzA9MDugOaA3hjVodHRwOi8vY2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9z
# aXRvcnkvZ2Ryb290LmNybDBLBgNVHSAERDBCMEAGBFUdIAAwODA2BggrBgEFBQcC
# ARYqaHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNvbS9yZXBvc2l0b3J5MA4G
# A1UdDwEB/wQEAwIBBjANBgkqhkiG9w0BAQUFAAOCAQEA0obA7L35obZn7mYLogY6
# BFCOFXKsSnSVU8s3y0RJ7weQazPZlvCUVqUTMAU8hTIhe8nHCqgkpJDeRtMlIxQD
# Z8IQ1m8PXXt6zJ/FWCrBxJ4hqFrzrKRG857kY8svkKQpKQHZciwp3zcBJ7xP7mjT
# IY/As+T1Ce3SEKpTtL7wzFkL1juWHJUkSd/O7P2nSJEURQ46Nm/aRbNFokHJ1NdE
# Tj65dHbVohNVLMaHo7WZrAaEh391Bvy/FEwOzG7E3z23EnH06PFRQCIoSeAdS4eo
# NMwGot0SWtGGNmQDNW9vd27r8oVQmF6rA1OtkSNjHxaczbmyBWM64fRoGxcFNZVT
# 7jCCBUYwggQuoAMCAQICBwQe+Z3rVhEwDQYJKoZIhvcNAQEFBQAwgcoxCzAJBgNV
# BAYTAlVTMRAwDgYDVQQIEwdBcml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRow
# GAYDVQQKExFHb0RhZGR5LmNvbSwgSW5jLjEzMDEGA1UECxMqaHR0cDovL2NlcnRp
# ZmljYXRlcy5nb2RhZGR5LmNvbS9yZXBvc2l0b3J5MTAwLgYDVQQDEydHbyBEYWRk
# eSBTZWN1cmUgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkxETAPBgNVBAUTCDA3OTY5
# Mjg3MB4XDTE0MDYxMjE5NDEyM1oXDTE2MDYxMjE5NDEyM1owdDELMAkGA1UEBhMC
# VVMxFjAUBgNVBAgTDU1hc3NhY2h1c2V0dHMxDzANBgNVBAcTBkJvc3RvbjEdMBsG
# A1UEChMUUHJveHkgTmV0d29ya3MsIEluYy4xHTAbBgNVBAMTFFByb3h5IE5ldHdv
# cmtzLCBJbmMuMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAtv978VTM
# H2dMJtBo61fkYAGjZRWmOihwCeu/hZf535rH+8GC0OJXZDxYvXzSlFVnvrS/6R1W
# +SK0xL2yqgec3Uvv7R/CftM/dqhqjLMlAv1Q+sf7n11QnkqzeJn1qGcg18pakHuU
# jmPTvY/2SMxMDQ0BMnz1JPoUXbuy7QFxpwX7WGyI2vFRf7NrGqnEny6L2Ze2vWPs
# idigKYVuNZRubdtTkjU9DBuBDJMDyPc7yvpwrL3TSaGWSE8zlG3u/VkA5T+KRD+2
# yJm0aAHs8katpf1XuTCO1cyLBQZlc/+XuYAmMOS+wvodrQpvYWJB0NzlOhhrE7BO
# CfPvHCqTEUOKxwIDAQABo4IBhDCCAYAwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAK
# BggrBgEFBQcDAzAOBgNVHQ8BAf8EBAMCB4AwMwYDVR0fBCwwKjAooCagJIYiaHR0
# cDovL2NybC5nb2RhZGR5LmNvbS9nZHM1LTE2LmNybDBTBgNVHSAETDBKMEgGC2CG
# SAGG/W0BBxcCMDkwNwYIKwYBBQUHAgEWK2h0dHA6Ly9jZXJ0aWZpY2F0ZXMuZ29k
# YWRkeS5jb20vcmVwb3NpdG9yeS8wgYAGCCsGAQUFBwEBBHQwcjAkBggrBgEFBQcw
# AYYYaHR0cDovL29jc3AuZ29kYWRkeS5jb20vMEoGCCsGAQUFBzAChj5odHRwOi8v
# Y2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvZ2RfaW50ZXJtZWRp
# YXRlLmNydDAfBgNVHSMEGDAWgBT9rGEyk2xF1uLuhV+auud2mWjM5zAdBgNVHQ4E
# FgQUqY92Vaco8oUA0CO79l2esDS5tgQwDQYJKoZIhvcNAQEFBQADggEBACMx2cCE
# kirZLB7AtAc3rhVXup7MG3Ogf4kTNnh1VY6hwEtf7c34B4u6R27SB7Sz+qrtA3Ui
# Ll3Q57Xh4oGladL6I6DYVE7XA33QIOt64/mMK749Ke+wZiFW1Js8pXu9v4GS2KXo
# wN9QIu2YGdQL/FH3dWlcLFs3BDbn75ezkF6QDjOnuFaja8ZSqehoakOV+bcRMaCW
# XqdrDM6ZsuXHR5+DeIC6kdTm0jpul3DK2WWUvX77qyewHW84H34skmw3C7GBoAAd
# nIOT8H1/9sut6P7xN2Go+n5FPMgq2b0uISojnxyokYqZ4bxIsn8nbNABDFQ09JrT
# 8RhpKTCfUlGdwyExggSHMIIEgwIBATCB1jCByjELMAkGA1UEBhMCVVMxEDAOBgNV
# BAgTB0FyaXpvbmExEzARBgNVBAcTClNjb3R0c2RhbGUxGjAYBgNVBAoTEUdvRGFk
# ZHkuY29tLCBJbmMuMTMwMQYDVQQLEypodHRwOi8vY2VydGlmaWNhdGVzLmdvZGFk
# ZHkuY29tL3JlcG9zaXRvcnkxMDAuBgNVBAMTJ0dvIERhZGR5IFNlY3VyZSBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eTERMA8GA1UEBRMIMDc5NjkyODcCBwQe+Z3rVhEw
# CQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# IwYJKoZIhvcNAQkEMRYEFLUW05Ce2Ny7/YES5m5L0p9fwEn/MA0GCSqGSIb3DQEB
# AQUABIIBALAOJELiv72yFV5sJiMYYotEIU4dnQnKdmtfUINBwDJN5DFYfQ912mOe
# 7dG0/763GU3qFErNj1lgI+m2BGa0kBZG7H3XXIEXdjpWRP4on246oNCMrXUFxP79
# KQgAYOKjGM6SvcdfuFEANbWRthqX3J4F8JvQhxefWNSZnVZh3psaOqpbwFnXbuUu
# 7e0jmwVlESOxv/1Zzt6q/okfMjrb6iWpFzVnNQhOkoNU3RHWiDQH98mJi5DQqbwn
# gjPG9hbTinPnpxyaQbjm3PJUWKZTFIjiqe8Q8ihuWWUlNRfKeuFhuHPhorQ7YVVm
# q8Xp85tq4RxDumDeJ3Vq9pVGlSxzhVShggILMIICBwYJKoZIhvcNAQkGMYIB+DCC
# AfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9y
# YXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMg
# Q0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0wGAYJKoZIhvcN
# AQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTYwMjI5MjA0NTA0WjAj
# BgkqhkiG9w0BCQQxFgQUIMXrg8dtjrCgXYKluvg+QMXOcQIwDQYJKoZIhvcNAQEB
# BQAEggEAcDEl+wIWu/+wTlI5pcRuuDiPYIVzCsGeXuDxxWZVSRHY6oyOkUFxgP2N
# wpZp43G9FJH/3k59vv/+bQ7V4JfFPhPKKC3MVbb16JTxyxH6o+ZXU28C0n9HhQoi
# dkt84SVyDg4j3XWZG3GjuVHyjciiE791oSB8Ij7fAOBznrFhkijv2Pdz1z2efBid
# tYCnFDlWklrVsJo99Gns7Pa+LXIcwkVdZhyEOLlduDYyR38TLIn6bkVjgoRO87iW
# eJjFX76s5LxmQyX1Xh1uWioedJAkv9speinJlhTsOTwt/oSvMrOEgiw0U5/6PSSR
# k3nRL7MFgJl/P8qHWrMrZR+vJXASVw==
# SIG # End signature block
