﻿##############################################################################
##
## Power Shell Module for managing Proxy Gateway Security
##
## Exported functions:
##   Find-AllUserACEs
##   Remove-AllUserACEs
##
##   Add-UserToGWSDataServicesSecurity
##   Add-UserToGWSSettingsSecurity
##   Add-UserToGWSOperationSecurity
##   Add-UserToGWSGroupSecurity
##   Add-UserToGWSHostSecurity
##   
##   Remove-UserFromGWSDataServicesSecurity
##   Remove-UserFromGWSSettingsSecurity
##   Remove-UserFromGWSOperationSecurity
##   Remove-UserFromGWSGroupSecurity
##   Remove-UserFromGWSHostSecurity
##   
##   
## Exported examples:
##
##
##
##############################################################################


# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Constants
Import-Module Proxy-ObjectSec
Import-Module Proxy-Gateway-Utils


<#

.FUNCTION NAME

Find-AllUserACEs

.SYNOPSIS

Finds and prints all security settings for the given User Identity in the Proxy Gateway Server

.RELATED

.EXAMPLE

PS > Find-AllUserACEs -prxGWS $gws -Domain "ourdomain.com" -Name "alex"

#>
function Find-AllUserACEs(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    Write-Host ("-" * 40)
    Write-Host "Security Rights for $Domain\\$Name"
    Write-Host ("-" * 40)
    Write-Host 

    Write-Host "Data Services Security"
    Write-Host ("-" * 20)
    $rights = Find-UserACEs -PrxSecurityObject $prxGWS.security -Domain $Domain -Name $Name -SecRightsMap $gatewaySecRightsMap
    if($rights) { $rights | foreach { Write-Host "`t$_" } }
    Write-Host 

    Write-Host "Settings Security"
    Write-Host ("-" * 20)
    $rights = Find-UserACEs -PrxSecurityObject $prxGWS.settings.security -Domain $Domain -Name $Name -SecRightsMap $settingsSecRightsMap
    if($rights) { $rights | foreach { Write-Host "`t$_" } }
    Write-Host 
    
    Write-Host "Operation Security"
    Write-Host ("-" * 20)
    $rights = Find-UserACEs -PrxSecurityObject $prxGWS.operationSecurity -Domain $Domain -Name $Name -SecRightsMap $operationSecRightsMap
    if($rights) { $rights | foreach { Write-Host "`t$_" } }
    Write-Host 
    
    Write-Host "Group Security"
    Write-Host ("-" * 20)
    foreach($group in $prxGWS.getGroups())
    {
        if(-not $group) { continue }
        
        $groupRights = Find-UserACEs -PrxSecurityObject $group.security -Domain $Domain -Name $Name -SecRightsMap $groupSecRightsMap 
        $sessionRights = Find-UserACEs -PrxSecurityObject $group.sessionSecurity -Domain $Domain -Name $Name -SecRightsMap $sessionSecRightsMap
        $hostRights = Find-UserACEs -PrxSecurityObject $group.hostSecurity -Domain $Domain -Name $Name -SecRightsMap $hostSecRightsMap
        if(-not $groupRights -and -not $sessionRights -and -not $hostRights)
        {
            continue
        }
        
        Write-Host "Group: $($group.groupName), $($group.groupID)"
        
        if($groupRights)
        {
            Write-Host "Group Security: "
            $groupRights | foreach { Write-Host "`t$_" }
        }
        
        if($sessionRights)
        {
            Write-Host "Session Security: "
            $sessionRights | foreach { Write-Host "`t$_" }
        }        
        
        if($hostRights)
        {
            Write-Host "Host Security: "
            $hostRights | foreach { Write-Host "`t$_" }
        }
        Write-Host 
    }
    
    Write-Host "Host Security"
    Write-Host ("-" * 20)
    $hosts = $prxGWS.getWorkstations() | where { $_.machine -ne "virtual" }
    foreach($host in $hosts)
    {
        if(-not $host) { continue }
        
        $hostRights = Find-UserACEs -PrxSecurityObject $host.security -Domain $Domain -Name $Name -SecRightsMap $hostSecRightsMap
        $sessionRights = Find-UserACEs -PrxSecurityObject $host.sessionSecurity -Domain $Domain -Name $Name -SecRightsMap $sessionSecRightsMap
        if(-not $hostRights -and -not $sessionRights)
        {
            continue
        }
        
        Write-Host "Host: $($host.name), $($host.key)"
        
        if($hostRights)
        {
            Write-Host "Host Security: "
            $hostRights | foreach { Write-Host "`t$_" }
        }
        
        if($sessionRights)
        {
            Write-Host "Session Security: "
            $sessionRights | foreach { Write-Host "`t$_" }
        }
        Write-Host 
    }

    Write-Host "User Security"
    Write-Host ("-" * 20)
    $users = $prxGWS.getUsers()
    foreach($user in $users)
    {
        if(-not $user) { continue }
        
        $hostRights = Find-UserACEs -PrxSecurityObject $user.security -Domain $Domain -Name $Name -SecRightsMap $hostSecRightsMap
        $sessionRights = Find-UserACEs -PrxSecurityObject $user.sessionSecurity -Domain $Domain -Name $Name -SecRightsMap $sessionSecRightsMap
        if(-not $hostRights -and -not $sessionRights)
        {
            continue
        }
        
        Write-Host "User: $($user.name)"
        
        if($hostRights)
        {
            Write-Host "Host Security: "
            $hostRights | foreach { Write-Host "`t$_" }
        }
        
        if($sessionRights)
        {
            Write-Host "Session Security: "
            $sessionRights | foreach { Write-Host "`t$_" }
        }
        Write-Host 
    }
    
}

<#

.FUNCTION NAME

Remove-AllUserACEs

.SYNOPSIS

Removes all security settings for the given User Identity from the Proxy Gateway Server

.RELATED

.EXAMPLE

PS > Remove-AllUserACEs -prxGWS $gws -Domain "ourdomain.com" -Name "alex"

#>
function Remove-AllUserACEs(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return
    }
    
    Write-Host ("-" * 40)
    Write-Host "Removing Security Rights for $Domain\\$Name"
    Write-Host ("-" * 40)
    Write-Host 

    if(Remove-UserACEs -PrxSecurityObject $prxGWS.security -Domain $Domain -Name $Name)
    {
        Write-Host "Removed From Data Services Security"
    }

    if(Remove-UserACEs -PrxSecurityObject $prxGWS.settings.security -Domain $Domain -Name $Name)
    {
        Write-Host "Removed From Settings Security"
    }
    
    if(Remove-UserACEs -PrxSecurityObject $prxGWS.operationSecurity -Domain $Domain -Name $Name)
    {
        Write-Host "Removed From Operation Security"
    }
    
    foreach($group in $prxGWS.getGroups())
    {
        if(-not $group) { continue }
        
        $removed = $false
        $removed = (Remove-UserACEs -PrxSecurityObject $group.security -Domain $Domain -Name $Name) -or $removed
        $removed = (Remove-UserACEs -PrxSecurityObject $group.sessionSecurity -Domain $Domain -Name $Name) -or $removed
        $removed = (Remove-UserACEs -PrxSecurityObject $group.hostSecurity -Domain $Domain -Name $Name) -or $removed
        if($removed)
        {
            Write-Host "Removed From Group: $($group.groupName), $($group.groupID) Security"
        }
    }
    
    $hosts = $prxGWS.getWorkstations() | where { $_.machine -ne "virtual" }
    foreach($host in $hosts)
    {
        if(-not $host) { continue }
        
        $removed = $false
        $removed = (Remove-UserACEs -PrxSecurityObject $host.security -Domain $Domain -Name $Name) -or $removed
        $removed = (Remove-UserACEs -PrxSecurityObject $host.sessionSecurity -Domain $Domain -Name $Name) -or $removed
        if($removed)
        {
            Write-Host "Removed From Host: $($host.name), $($host.key) Security"
        }
    }

    $users = $prxGWS.getUsers()
    foreach($user in $users)
    {
        if(-not $user) { continue }
        
        $removed = $false
        $removed = (Remove-UserACEs -PrxSecurityObject $user.security -Domain $Domain -Name $Name) -or $removed
        $removed = (Remove-UserACEs -PrxSecurityObject $user.sessionSecurity -Domain $Domain -Name $Name) -or $removed
        if($removed)
        {
            Write-Host "Removed From User: $($user.name) Security"
        }
    }
}

<#

.FUNCTION NAME

Add-UserToGWSDataServicesSecurity

.SYNOPSIS

Add new user to Data Services Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Add-UserToGWSDataServicesSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -AccessMask ($gatewayModifyInformation -bor $gatewayEditSecurity)

#>
function Add-UserToGWSDataServicesSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [int]$AccessMask = $(throw "Please provide Access Mask"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.security -Domain $Domain -Name $Name -AccessMask $AccessMask
}

<#

.FUNCTION NAME

Add-UserToGWSSettingsSecurity

.SYNOPSIS

Add new user to Settings Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Add-UserToGWSSettingsSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -AccessMask ($settingsModifySettings -bor $settingsEditSecurity)

#>
function Add-UserToGWSSettingsSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [int]$AccessMask = $(throw "Please provide Access Mask"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.settings.security -Domain $Domain -Name $Name -AccessMask $AccessMask
}

<#

.FUNCTION NAME

Add-UserToGWSOperationSecurity

.SYNOPSIS

Add new user to Operation Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Add-UserToGWSOperationSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -AccessMask ($operationViewDiagnostics -bor $operationEditSecurity)

#>
function Add-UserToGWSOperationSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [int]$AccessMask = $(throw "Please provide Access Mask"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.operationSecurity -Domain $Domain -Name $Name -AccessMask $AccessMask
}

<#

.FUNCTION NAME

Add-UserToGWSGroupSecurity

.SYNOPSIS

Add new user to Group Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Add-UserToGWSGroupSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -GroupName "Support" -HostAccessMask $hostFullUser -SessionAccessMask $sessionPlayRecordedSession
or
PS > Remove-UserFromGWSGroupSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -GroupID "{76837334-DA68-42F2-835F-7910661285B5}" -HostAccessMask $hostFullUser -SessionAccessMask $sessionPlayRecordedSession

#>
function Add-UserToGWSGroupSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [string]$GroupName = "",
    [string]$GroupID = "",
    [int]$GroupAccessMask = 0,
    [int]$HostAccessMask = 0,
    [int]$SessionAccessMask = 0)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    if(-not $GroupAccessMask -and -not $SessionAccessMask -and -not $HostAccessMask)
    {
        Write-Error -message "Error: No access mask specified in parameters." 
        return $false
    }
    
    if(-not $GroupName -and -not $GroupID)
    {
        Write-Error -message "Error: Either GroupName or GroupID should be specified." 
        return $false
    }
    
    $group = $prxGWS.getGroups() | where { $_.groupName -eq $GroupName -or $_.groupID -eq $GroupID }
    if(-not $group)
    {
        Write-Error -message "Error: Group(Name: $GroupName, ID: $GroupID) not found." 
        return $false
    }
    
    if($GroupAccessMask -and -not (Add-AccessAllowed -PrxSecurityObject $group.security -Domain $Domain -Name $Name -AccessMask $GroupAccessMask))
    {
    	Write-Error -message "Error: Add-AccessAllowed to Group Security failed." 
        return $false
    }
    
    if($HostAccessMask -and -not (Add-AccessAllowed -PrxSecurityObject $group.hostSecurity -Domain $Domain -Name $Name -AccessMask $HostAccessMask))
    {
    	Write-Error -message "Error: Add-AccessAllowed to Group's Host Security failed." 
        return $false
    }
    
    if($SessionAccessMask -and -not (Add-AccessAllowed -PrxSecurityObject $group.sessionSecurity -Domain $Domain -Name $Name -AccessMask $SessionAccessMask))
    {
    	Write-Error -message "Error: Add-AccessAllowed to Group's Session Security failed." 
        return $false
    }
    
   	return $true
}

<#

.FUNCTION NAME

Add-UserToGWSHostSecurity

.SYNOPSIS

Add new user to Host Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Add-UserToGWSHostSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -HostName "WKS-BOSTON-1599" -HostAccessMask $hostFullUser -SessionAccessMask $sessionPlayRecordedSession
or
PS > Add-UserToGWSHostSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -HostKey "w={00dbbb0f-8654-11e0-a5d6-005056c00008}" -HostAccessMask $hostFullUser -SessionAccessMask $sessionPlayRecordedSession

#>
function Add-UserToGWSHostSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [string]$HostName = "",
    [string]$HostKey = "",
    [int]$HostAccessMask = 0,
    [int]$SessionAccessMask = 0)
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    if(-not $SessionAccessMask -and -not $HostAccessMask)
    {
        Write-Error -message "Error: No access mask specified in parameters." 
        return $false
    }
    
    if(-not $HostName -and -not $HostKey)
    {
        Write-Error -message "Error: Either HostName or HostKey should be specified." 
        return $false
    }
    
    $host = $prxGWS.getWorkstations() | where { $_.machine -ne "virtual" -and ($_.name -eq $HostName -or $_.key -eq $HostKey) }
    if(-not $host)
    {
        Write-Error -message "Error: Host(Name: $HostName, key: $HostKey) not found." 
        return $false
    }
    
    if($host.count -gt 1)
    {
        Write-Error -message "Error: More then one Host(Name: $HostName, key: $HostKey) found." 
        return $false
    }
    
    if($HostAccessMask -and -not (Add-AccessAllowed -PrxSecurityObject $host.security -Domain $Domain -Name $Name -AccessMask $HostAccessMask))
    {
    	Write-Error -message "Error: Add-AccessAllowed to Host Security failed." 
        return $false
    }
    
    if($SessionAccessMask -and -not (Add-AccessAllowed -PrxSecurityObject $host.sessionSecurity -Domain $Domain -Name $Name -AccessMask $SessionAccessMask))
    {
    	Write-Error -message "Error: Add-AccessAllowed to Host's Session Security failed." 
        return $false
    }
    
   	return $true
}

<#

.FUNCTION NAME

Remove-UserFromGWSDataServicesSecurity

.SYNOPSIS

Removes user from Data Services Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Remove-UserFromGWSDataServicesSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex"

#>
function Remove-UserFromGWSDataServicesSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Remove-UserACEs -PrxSecurityObject $prxGWS.security -Domain $Domain -Name $Name
}

<#

.FUNCTION NAME

Remove-UserFromGWSSettingsSecurity

.SYNOPSIS

Removes user from Settings Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Remove-UserFromGWSSettingsSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex"

#>
function Remove-UserFromGWSSettingsSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Remove-UserACEs -PrxSecurityObject $prxGWS.settings.security -Domain $Domain -Name $Name
}

<#

.FUNCTION NAME

Remove-UserFromGWSOperationSecurity

.SYNOPSIS

Removes user from Operation Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Remove-UserFromGWSOperationSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex"

#>
function Remove-UserFromGWSOperationSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    return Remove-UserACEs -PrxSecurityObject $prxGWS.operationSecurity -Domain $Domain -Name $Name
}

<#

.FUNCTION NAME

Remove-UserFromGWSGroupSecurity

.SYNOPSIS

Removes user from Group Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Remove-UserFromGWSGroupSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -GroupName "Support"
or
PS > Remove-UserFromGWSGroupSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -GroupID "{76837334-DA68-42F2-835F-7910661285B5}"

#>
function Remove-UserFromGWSGroupSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [string]$GroupName = "",
    [string]$GroupID = "")
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    if(-not $GroupName -and -not $GroupID)
    {
        Write-Error -message "Error: Either GroupName or GroupID should be specified." 
        return $false
    }
    
    $group = $prxGWS.getGroups() | where { $_.groupName -eq $GroupName -or $_.groupID -eq $GroupID }
    if(-not $group)
    {
        Write-Error -message "Error: Group(Name: $GroupName, ID: $GroupID) not found." 
        return $false
    }

    $removed = $false
    $removed = (Remove-UserACEs -PrxSecurityObject $group.security -Domain $Domain -Name $Name) -or $removed
    $removed = (Remove-UserACEs -PrxSecurityObject $group.sessionSecurity -Domain $Domain -Name $Name) -or $removed
    $removed = (Remove-UserACEs -PrxSecurityObject $group.hostSecurity -Domain $Domain -Name $Name) -or $removed
    return $removed
}

<#

.FUNCTION NAME

Remove-UserFromGWSHostSecurity

.SYNOPSIS

Removes user from Host Security of Proxy Gateway Server.

.RELATED

.EXAMPLE

PS > Remove-UserFromGWSHostSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -HostName "WKS-BOSTON-1599"
or
PS > Remove-UserFromGWSHostSecurity -prxGWS $gws -Domain "ourdomain.com" -Name "alex" -HostKey "w={00dbbb0f-8654-11e0-a5d6-005056c00008}"

#>
function Remove-UserFromGWSHostSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"),
    [string]$HostName = "",
    [string]$HostKey = "")
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    if(-not $HostName -and -not $HostKey)
    {
        Write-Error -message "Error: Either HostName or HostKey should be specified." 
        return $false
    }
    
    $host = $prxGWS.getWorkstations() | where { $_.machine -ne "virtual" -and ($_.name -eq $HostName -or $_.key -eq $HostKey) }
    if(-not $host)
    {
        Write-Error -message "Error: Host(Name: $HostName, key: $HostKey) not found." 
        return $false
    }
    
    if($host.count -gt 1)
    {
        Write-Error -message "Error: More then one Host(Name: $HostName, key: $HostKey) found." 
        return $false
    }

    $removed = $false
    $removed = (Remove-UserACEs -PrxSecurityObject $host.security -Domain $Domain -Name $Name) -or $removed
    $removed = (Remove-UserACEs -PrxSecurityObject $host.sessionSecurity -Domain $Domain -Name $Name) -or $removed
   	return $removed
}

#
# Module exports
#
Export-ModuleMember -Function  Find-*, Add-*, Remove-*

# SIG # Begin signature block
# MIIXcgYJKoZIhvcNAQcCoIIXYzCCF18CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUZYR3HFcEyvPJOOJNTqp2FxEn
# eTugghLuMIIDnzCCAoegAwIBAgIQeaKlhfnRFUIT2bg+9raN7TANBgkqhkiG9w0B
# AQUFADBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xKzAp
# BgNVBAMTIlZlcmlTaWduIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EwHhcNMTIw
# NTAxMDAwMDAwWhcNMTIxMjMxMjM1OTU5WjBiMQswCQYDVQQGEwJVUzEdMBsGA1UE
# ChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xNDAyBgNVBAMTK1N5bWFudGVjIFRpbWUg
# U3RhbXBpbmcgU2VydmljZXMgU2lnbmVyIC0gRzMwgZ8wDQYJKoZIhvcNAQEBBQAD
# gY0AMIGJAoGBAKlZZnTaPYp9etj89YBEe/5HahRVTlBHC+zT7c72OPdPabmx8LZ4
# ggqMdhZn4gKttw2livYD/GbT/AgtzLVzWXuJ3DNuZlpeUje0YtGSWTUUi0WsWbJN
# JKKYlGhCcp86aOJri54iLfSYTprGr7PkoKs8KL8j4ddypPIQU2eud69RAgMBAAGj
# geMwgeAwDAYDVR0TAQH/BAIwADAzBgNVHR8ELDAqMCigJqAkhiJodHRwOi8vY3Js
# LnZlcmlzaWduLmNvbS90c3MtY2EuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMI
# MDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNp
# Z24uY29tMA4GA1UdDwEB/wQEAwIHgDAeBgNVHREEFzAVpBMwETEPMA0GA1UEAxMG
# VFNBMS0zMB0GA1UdDgQWBBS0t/GJSSZg52Xqc67c0zjNv1eSbzANBgkqhkiG9w0B
# AQUFAAOCAQEAHpiqJ7d4tQi1yXJtt9/ADpimNcSIydL2bfFLGvvV+S2ZAJ7R55uL
# 4T+9OYAMZs0HvFyYVKaUuhDRTour9W9lzGcJooB8UugOA9ZresYFGOzIrEJ8Byyn
# PQhm3ADt/ZQdc/JymJOxEdaP747qrPSWUQzQjd8xUk9er32nSnXmTs4rnykr589d
# nwN+bid7I61iKWavkugszr2cf9zNFzxDwgk/dUXHnuTXYH+XxuSqx2n1/M10rCyw
# SMFQTnBWHrU1046+se2svf4M7IV91buFZkQZXZ+T64K6Y57TfGH/yBvZI1h/MKNm
# oTkmXpLDPMs3Mvr1o43c1bCj6SU2VdeB+jCCA8QwggMtoAMCAQICEEe/GZXfjVJG
# Q/fbbUgNMaQwDQYJKoZIhvcNAQEFBQAwgYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQI
# EwxXZXN0ZXJuIENhcGUxFDASBgNVBAcTC0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZU
# aGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBDZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZU
# aGF3dGUgVGltZXN0YW1waW5nIENBMB4XDTAzMTIwNDAwMDAwMFoXDTEzMTIwMzIz
# NTk1OVowUzELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSsw
# KQYDVQQDEyJWZXJpU2lnbiBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBMIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqcqypMzNIK8KfYmsh3XwtE7x38EP
# v2dhvaNkHNq7+cozq4QwiVh+jNtr3TaeD7/R7Hjyd6Z+bzy/k68Numj0bJTKvVIt
# q0g99bbVXV8bAp/6L2sepPejmqYayALhf0xS4w5g7EAcfrkN3j/HtN+HvV96ajEu
# A5mBE6hHIM4xcw1XLc14NDOVEpkSud5oL6rm48KKjCrDiyGHZr2DWFdvdb88qiaH
# XcoQFTyfhOpUwQpuxP7FSt25BxGXInzbPifRHnjsnzHJ8eYiGdvEs0dDmhpfoB6Q
# 5F717nzxfatiAY/1TQve0CJWqJXNroh2ru66DfPkTdmg+2igrhQ7s4fBuwIDAQAB
# o4HbMIHYMDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3Au
# dmVyaXNpZ24uY29tMBIGA1UdEwEB/wQIMAYBAf8CAQAwQQYDVR0fBDowODA2oDSg
# MoYwaHR0cDovL2NybC52ZXJpc2lnbi5jb20vVGhhd3RlVGltZXN0YW1waW5nQ0Eu
# Y3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIBBjAkBgNVHREE
# HTAbpBkwFzEVMBMGA1UEAxMMVFNBMjA0OC0xLTUzMA0GCSqGSIb3DQEBBQUAA4GB
# AEpr+epYwkQcMYl5mSuWv4KsAdYcTM2wilhu3wgpo17IypMT5wRSDe9HJy8AOLDk
# yZNOmtQiYhX3PzchT3AxgPGLOIez6OiXAP7PVZZOJNKpJ056rrdhQfMqzufJ2V7d
# uyuFPrWdtdnhV/++tMV+9c8MnvCX/ivTO1IbGzgn9z9KMIIFcTCCBFmgAwIBAgIQ
# XM5xW+LS1ji4A9NViFFAMDANBgkqhkiG9w0BAQUFADCBtDELMAkGA1UEBhMCVVMx
# FzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVz
# dCBOZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cu
# dmVyaXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3Mg
# MyBDb2RlIFNpZ25pbmcgMjAxMCBDQTAeFw0xMjA2MTUwMDAwMDBaFw0xNDA3MDky
# MzU5NTlaMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UECBMNTWFzc2FjaHVzZXR0czEP
# MA0GA1UEBxMGQm9zdG9uMR0wGwYDVQQKFBRQcm94eSBOZXR3b3JrcywgSW5jLjE+
# MDwGA1UECxM1RGlnaXRhbCBJRCBDbGFzcyAzIC0gTWljcm9zb2Z0IFNvZnR3YXJl
# IFZhbGlkYXRpb24gdjIxHTAbBgNVBAMUFFByb3h5IE5ldHdvcmtzLCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuORukVR3ArkRbmIr7uMEWljY
# GxNuA2MUK0RKXLO8sswGaEPlSq2KeysSQZVScPGcJtkNj+lIsZ5ECM+bYdAJ4Svv
# BTEqt0HKHsovnv7UKFUCoDOHvVZThWA7+58iFrbMMa/QrPWaqBqYMBGyCtRRZjwf
# pE5kWKe617Kcz0w+myqJv/FmOg7mXe5/j25U21/C48q6uE+CzTvm/fv/DhCOZrMH
# ts641LnzX5I3SNv24pMZ0eUU7hGG9J42u+m3D/yC98DkzWIfYnNCcM2COQCkx2FR
# 6MoB2ny6PN+YpdgF2nD71t2LuJMDxVF6PGbE9b+aNMfraRq1rwPfT8BzPwmavQID
# AQABo4IBezCCAXcwCQYDVR0TBAIwADAOBgNVHQ8BAf8EBAMCB4AwQAYDVR0fBDkw
# NzA1oDOgMYYvaHR0cDovL2NzYzMtMjAxMC1jcmwudmVyaXNpZ24uY29tL0NTQzMt
# MjAxMC5jcmwwRAYDVR0gBD0wOzA5BgtghkgBhvhFAQcXAzAqMCgGCCsGAQUFBwIB
# FhxodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMHEGCCsGAQUFBwEBBGUwYzAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVy
# aXNpZ24uY29tMDsGCCsGAQUFBzAChi9odHRwOi8vY3NjMy0yMDEwLWFpYS52ZXJp
# c2lnbi5jb20vQ1NDMy0yMDEwLmNlcjAfBgNVHSMEGDAWgBTPmanqeyb0S8mOj9fw
# BSbv49KnnTARBglghkgBhvhCAQEEBAMCBBAwFgYKKwYBBAGCNwIBGwQIMAYBAQAB
# Af8wDQYJKoZIhvcNAQEFBQADggEBABDAtzEM2dWEt6wcvi3fSe9Y+hwIS0Oi6Rmf
# QkWMWwHKXvZCOOzuq2i2ywCXnyPz+Fb7waMen0GK2DkfNOwjbjO3yKfDpzv/MHTY
# 1pQjbyf2ySqrRJZ1iqaTDdTe/dhz2PEnp3wIMCM3wX5OCfHEKDUx4BYcLwxaEMAY
# V0j/wJ/vIBdpbDG3GfBUpeRavYjYAEIau7HCMN7lKvWODP09JaePLwfqNzDKB+GY
# cjiEI0gZMY2U4gJ5VygYHGRQslgZ+gVqGg9A6wqWSjqLXBy8C5CRNQmds+e6081Y
# b3WvPbwtI1lGTNmJDCTkJuXDk2S5LDkOEg4DpilD1nN7MiqzRf0wggYKMIIE8qAD
# AgECAhBSAOWqJVb8GobtlsnUSzPHMA0GCSqGSIb3DQEBBQUAMIHKMQswCQYDVQQG
# EwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWdu
# IFRydXN0IE5ldHdvcmsxOjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMu
# IC0gRm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENs
# YXNzIDMgUHVibGljIFByaW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# NTAeFw0xMDAyMDgwMDAwMDBaFw0yMDAyMDcyMzU5NTlaMIG0MQswCQYDVQQGEwJV
# UzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRy
# dXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3
# dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFz
# cyAzIENvZGUgU2lnbmluZyAyMDEwIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA9SNLXqXXirsy6dRX9+/kxyZ+rRmY/qidfZT2NmsQ13WBMH8EaH/L
# K3UezR0IjN9plKc3o5x7gOCZ4e43TV/OOxTuhtTQ9Sc1vCULOKeMY50Xowilq7D7
# zWpigkzVIdob2fHjhDuKKk+FW5ABT8mndhB/JwN8vq5+fcHd+QW8G0icaefApDw8
# QQA+35blxeSUcdZVAccAJkpAPLWhJqkMp22AjpAle8+/PxzrL5b65Yd3xrVWsno7
# VDBTG99iNP8e0fRakyiF5UwXTn5b/aSTmX/fze+kde/vFfZH5/gZctguNBqmtKdM
# fr27Tww9V/Ew1qY2jtaAdtcZLqXNfjQtiQIDAQABo4IB/jCCAfowEgYDVR0TAQH/
# BAgwBgEB/wIBADBwBgNVHSAEaTBnMGUGC2CGSAGG+EUBBxcDMFYwKAYIKwYBBQUH
# AgEWHGh0dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9jcHMwKgYIKwYBBQUHAgIwHhoc
# aHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYTAOBgNVHQ8BAf8EBAMCAQYwbQYI
# KwYBBQUHAQwEYTBfoV2gWzBZMFcwVRYJaW1hZ2UvZ2lmMCEwHzAHBgUrDgMCGgQU
# j+XTGoasjY5rw8+AatRIGCx7GS4wJRYjaHR0cDovL2xvZ28udmVyaXNpZ24uY29t
# L3ZzbG9nby5naWYwNAYDVR0fBC0wKzApoCegJYYjaHR0cDovL2NybC52ZXJpc2ln
# bi5jb20vcGNhMy1nNS5jcmwwNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhho
# dHRwOi8vb2NzcC52ZXJpc2lnbi5jb20wHQYDVR0lBBYwFAYIKwYBBQUHAwIGCCsG
# AQUFBwMDMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBWZXJpU2lnbk1QS0ktMi04
# MB0GA1UdDgQWBBTPmanqeyb0S8mOj9fwBSbv49KnnTAfBgNVHSMEGDAWgBR/02Wn
# wt3su/AwCfNDOfoCrzMxMzANBgkqhkiG9w0BAQUFAAOCAQEAViLmNKTEYctIuQGt
# VqhkD9mMkcS7zAzlrXqgIn/fRzhKLWzRf3EafOxwqbHwT+QPDFP6FV7+dJhJJIWB
# JhyRFEewTGOMu6E01MZF6A2FJnMD0KmMZG3ccZLmRQVgFVlROfxYFGv+1KTteWsI
# DEFy5zciBgm+I+k/RJoe6WGdzLGQXPw90o2sQj1lNtS0PUAoj5sQzyMmzEsgy5Af
# XYxMNMo82OU31m+lIL006ybZrg3nxZr3obQhkTNvhuhYuyV8dA5Y/nUbYz/OMXyb
# jxuWnsVTdoRbnK2R+qztk7pdyCFTwoJTY68SDVCHERs9VFKWiiycPZIaCJoFLseT
# pUiR0zGCA+4wggPqAgEBMIHJMIG0MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVy
# aVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5
# BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20v
# cnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmlu
# ZyAyMDEwIENBAhBcznFb4tLWOLgD01WIUUAwMAkGBSsOAwIaBQCgeDAYBgorBgEE
# AYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwG
# CisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBQNWjg0
# a08/RcITNdy5xO0erPoZ9DANBgkqhkiG9w0BAQEFAASCAQBqrrRB1EFjx+AREm0A
# y9BE1k85BGBi91k0e386H+mh5jDciZQsbYEuiiF3nQaPURBCUN9VBDC1Zg735gAJ
# r5o0yV/gDMLr4xt7AQwucjMMrHgmBq1e3Q6Y+U/ksXtsiktiqZTIcOU9RWwMcAng
# VKCc1pO+NrquqmmZvHlHadX+F8iUDPKXtkyYEG+ihqdX/0LOBeLouIDTfOkLhMzm
# /kIQYZ9kOFeQQn+VOSyV0XXMEdCC+b8Zv4LMCG7ya9bXuFzt5z12f5lbzszPwko9
# YbdlGZvXWJaQMo6kl8m6JFpzl6lk97+1nqqTvx7XWKG1GjKCwvEJ04Rck001i+D0
# k5KJoYIBfzCCAXsGCSqGSIb3DQEJBjGCAWwwggFoAgEBMGcwUzELMAkGA1UEBhMC
# VVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSswKQYDVQQDEyJWZXJpU2lnbiBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBAhB5oqWF+dEVQhPZuD72to3tMAkGBSsO
# AwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEP
# Fw0xMjA4MDgyMjU1NTRaMCMGCSqGSIb3DQEJBDEWBBTE3xtTFdv7SBsW7oKMOKIS
# l9md8zANBgkqhkiG9w0BAQEFAASBgHDuC//dQXu9RkfgLccygAsTl1XdgVKP81bG
# 1HnX/CIZOB+sLYKKn7TnDHTlBjQUyBr7oKVDjhpFTM0exfNY6pV4vxVb/pXHg1Gn
# fFHOuAT+NWq94F4ZD4aUSYIoSIOkFshHqLOL2IbPXPGfOQ4dOW3wCl0ylmozcDzW
# rZ/jwHbj
# SIG # End signature block
