﻿##############################################################################
##
## Power Shell Module for managing Hosts
##
## Exported functions:
##  Connect-Host
##  Disconnect-Host
##  Test-HostConnection
##	Get-HostConnection
##
##############################################################################

# There can be only one active Host connection in one instance of the PS script. "activeHostConnection" holds it.
$script:activeHostConnection = $null

# Imports
Import-Module Proxy-Utils
Import-Module Proxy-ConnectionEvents
Import-Module Proxy-Utils


# Check if we are running in "STA" apartment. In this case the window's message notification between MTA and STA parts of ProxyGW object
# is not working unless message pump gets called explicitly.
if($Host.Runspace.ApartmentState -eq "STA") 
{
    Write-Warning "WARNING: You are running in STA Apartment mode.
This occurs in the PowerShell ISE, or by default in PowerShell v3 and later.
The Host Client's COM object will not function properly when running in
the STA. Specifically, if the network connection drops or fails, that will
not be reflected in the Host object state."
}


<#

.FUNCTION NAME

Connect-Host

.SYNOPSIS

This method establishes new Administrative connection to Proxy Host via Proxy Gateway Server. 
Method returns object representing established connection.
When user finish to work with the Host, Disconnect-Host should be called.

.RELATED
Disconnect-Host
Test-HostConnection
Get-HostConnection

.EXAMPLE

PS > $hostHA = Connect-Host -gateway "hostname.domain.com" -hostkey "0CC2B0E2-D6C3-4B88-BE97-38B3AA5A60C6" -protocol "TCP" -credentials "domain\user|password" -timeout 180
PS > $hostHA = Connect-Host -gateway "hostname.domain.com" -hostkey "0CC2B0E2-D6C3-4B88-BE97-38B3AA5A60C6" # connects with TCP protocol using current user identity of powershell and timeout 180sec
PS > 
PS > Disconnect-Host
PS >

#>
function Connect-Host([string]$gateway, [string]$hostkey, [string]$protocol='TCP', [string]$credentials='', [int]$timeout=180)
{
    if(Test-HostConnection)
    {
        Write-Error -message "Error: Already connected to the Host: $($script:activeHostConnection.host.currentStation.displayName). Only one connection per PS instance is allowed." `
            -Category ResourceExists -ErrorId $_ERROR_ID_AlreadyConnected
        
        return $true
    }
	
    # Create a new Proxy SDK object of Proxy.ProxyHA type
    $prxHA = New-Object -COM Proxy.ProxyHA
    if(-not $prxHA)
    { 
        Write-Error -message "Error: Failed to create Proxy.ProxyHA COM object." `
            -Category ObjectNotFound -ErrorId $_ERROR_ID_COMError
        
        return $false 
    }
    
    # Suppress UI messages from control
    $prxHA.displayMessages = $false
    
    # Register actions for connection events. The actions are; - issuing of a new
    # event with custom source which is a GUID and an argument indicating actual ProxyHA event, 
    # in order to wait for one event in the script code.
    $eventSource = [System.Guid]::NewGuid().Guid

    Register-ObjectEvent $prxHA onConnectSucceeded -SourceIdentifier ($eventSource + "|c")
    Register-ObjectEvent $prxHA onConnectFailed -SourceIdentifier ($eventSource + "|f")
    Register-ObjectEvent $prxHA onDisconnect  -SourceIdentifier ($eventSource + "|d")
	
    # Augment hostkey to the form 'w={<GUID>}'
    if($hostkey.StartsWith("{"))
    {
        $hostkey = "w=" + $hostkey
    }
    elseif (-not $hostkey.StartsWith("w={"))
    {
        $hostkey = "w={" + $hostkey
    }
    
    if(-not $hostkey.EndsWith("}"))
    {
        $hostkey = $hostkey + "}"
    }
    
    Write-Verbose "Connecting to Host:$gateway, $hostkey, $protocol, $credentials"
    
    # Initiate connection to the Proxy Host via the Gateway 
    $prxHA.connectToHostViaGateway($protocol, $gateway, $hostkey, $credentials)

    # Wait for connection to complete with $timeout
    $waitRes = Wait-EventWithMessageLoop ($eventSource + "*") -Timeout $timeout
    if($waitRes)
    {
      # Get the completion event
      $haEvent = Get-Event ($eventSource + "*") -ErrorAction:Continue
	  if($haEvent)
	  {
	      if($haEvent.SourceIdentifier -eq ($eventSource + "|c"))
	      {
	          Write-Verbose "Connection to: $hostkey - is successful."
	          
	          # Clean completion events from the queue
	          Remove-Event ($eventSource + "*")

	          # Remove event handlers 
	          (($eventSource + "|c"), ($eventSource + "|f"), ($eventSource + "|d")) | % { Unregister-Event -SourceIdentifier $_ -Force }
	          
	          # Remember new connection as an active connection
	          $script:activeHostConnection = New-Object PsObject -Property @{
	                  host = $prxHA;
	              }
	              
	          return $script:activeHostConnection.host
	      }
	      else
	      {
              $errorStr = "0x{0:X0}" -f $haEvent.SourceArgs
              Write-Error -message "Failed to connect to the Host: $hostkey. Error: $errorStr" `
	              -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
              # From here we will fall out to "Clean-up after errors" code section
	      }
	  }
	  else
	  {
          Write-Error -message "Failed to connect to the Host: $hostkey. Error: Unknown" `
              -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
          # From here we will fall out to "Clean-up after errors" code section
	  }
    }
    else
    {
        Write-Error -message "Connection to the Host timed out." `
            -Category ResourceUnavailable -ErrorId $_ERROR_ID_ConnectionFailed
        # From here we will fall out to "Clean-up after errors" code section
    }
    
    #
    # Clean-up after errors
    #
    
	# Clean completion events from the queue
	Remove-Event ($eventSource + "*") -ErrorAction:SilentlyContinue
	
    # Remove event handlers 
    (($eventSource + "|c"), ($eventSource + "|f"), ($eventSource + "|d")) | % { Unregister-Event -SourceIdentifier $_ -Force }

    # Issue disconnect on Host object in case we've hit timeout and underlying object is not disconnected yet
    $prxHA.disconnect()
    
	# Decrements the reference count of the specified Runtime Callable Wrapper (RCW) associated with the PrxHA COM object
	[System.Runtime.Interopservices.Marshal]::ReleaseComObject($prxHA) | Out-Null
    $prxHA = $null
    
    return $null
}

function Clean-State
{
    if($script:activeHostConnection) {
        if($script:activeHostConnection.host) {
   			# Decrements the reference count of the specified Runtime Callable Wrapper (RCW) associated with the PrxHA COM object
            [System.Runtime.Interopservices.Marshal]::ReleaseComObject($script:activeHostConnection.host) | Out-Null
	        $script:activeHostConnection.host = $null
        }
		
		$script:activeHostConnection = $null
    }
}

<#

.FUNCTION NAME

Disconnect-Host

.SYNOPSIS

This method disconnects from a Host to which a connection was previously established. 

.RELATED
Connect-Host
Test-HostConnection
Get-HostConnection

.EXAMPLE

PS >if(Test-HostConnection) { Disconnect-Host }

#>
function Disconnect-Host([int]$timeout=180)
{
    if(-not (Test-HostConnection))
    {
        Clean-State
        Write-Verbose "Not connected."
        return
    }

	# Register onDisconnect event
    $eventSource = [System.Guid]::NewGuid().Guid
    $null = Register-ObjectEvent $script:activeHostConnection.host onDisconnect -SourceIdentifier ($eventSource + "|d")
	
    $script:activeHostConnection.host.disconnect()
    
    $null = Wait-EventWithMessageLoop ($eventSource + "|d") -Timeout $timeout
	
    Remove-Event ($eventSource + "|d")
	Unregister-Event -SourceIdentifier ($eventSource + "|d") -Force

    Clean-State 
    
    Write-Verbose "Disconnected."
}

<#

.FUNCTION NAME

Test-HostConnection

.SYNOPSIS

Tests Administrative connection to Host 

.RELATED
Connect-Host
Disconnect-Host
Get-HostConnection

.EXAMPLE

PS >if(Test-HostConnection) { Disconnect-Host }

#>
function Test-HostConnection
{
    if($script:activeHostConnection `
	-and $script:activeHostConnection.host `
	-and $script:activeHostConnection.host.connectStatus -eq 1) # Where connectStatus('1') is: stateConnectedToService
    {
        return $true
    }

    return $false
}

<#

.FUNCTION NAME

Get-HostConnection

.SYNOPSIS

Returns current Host Administration Object or $null if there is no connection established. 

.RELATED
Connect-Host
Disconnect-Host
Test-HostConnection

.EXAMPLE

PS >if(Test-HostConnection) { Write-Host (Get-HostConnection).currentStation }

#>
function Get-HostConnection
{
    if(Test-HostConnection)
    {
        return $script:activeHostConnection.host
    }
    
    return $null
}

#
# Module exports
#
Export-ModuleMember -Function Connect-Host, Disconnect-Host, Test-HostConnection, Get-HostConnection


# SIG # Begin signature block
# MIIX4gYJKoZIhvcNAQcCoIIX0zCCF88CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUmt9ZmVRvUzx6QRX9+OIoMYDu
# eEGgghLFMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggTeMIIDxqADAgECAgIDATANBgkqhkiG9w0BAQUFADBjMQswCQYDVQQGEwJVUzEh
# MB8GA1UEChMYVGhlIEdvIERhZGR5IEdyb3VwLCBJbmMuMTEwLwYDVQQLEyhHbyBE
# YWRkeSBDbGFzcyAyIENlcnRpZmljYXRpb24gQXV0aG9yaXR5MB4XDTA2MTExNjAx
# NTQzN1oXDTI2MTExNjAxNTQzN1owgcoxCzAJBgNVBAYTAlVTMRAwDgYDVQQIEwdB
# cml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRowGAYDVQQKExFHb0RhZGR5LmNv
# bSwgSW5jLjEzMDEGA1UECxMqaHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNv
# bS9yZXBvc2l0b3J5MTAwLgYDVQQDEydHbyBEYWRkeSBTZWN1cmUgQ2VydGlmaWNh
# dGlvbiBBdXRob3JpdHkxETAPBgNVBAUTCDA3OTY5Mjg3MIIBIjANBgkqhkiG9w0B
# AQEFAAOCAQ8AMIIBCgKCAQEAxC3VFYycJkzsMjXrX7hZAVqmYYFZO3Bjq+PcPccq
# uMkz03nkOu08MCOEjrMwFLayh8M9lVQEnt+Z3QslHiHeZSl+NaipVOv29zI51CZV
# la3v+/5Yhtee9ACNjCoMvUIEzqc/BPbugPKq71KhaWbavhqtXdosZuoaa7vlGlFK
# AC9Ix5h12LkpyO74Zm0KnLPz/Hh8ovij8rXD87l6kcGn5iUunKjtEmVuavYSRFNw
# MJXDnCtYKz0IdEryvlGwv4fQTCdYa7U1xZ2vFzH4C4/urYE2BYkImM86ryWHwEnq
# p/1n90WOl8wUOeI2hbV+Gjf9FvZxEZp0MBb+E5SjP4QNTwIDAQABo4IBMjCCAS4w
# HQYDVR0OBBYEFP2sYTKTbEXW4u6FX5q653aZaMznMB8GA1UdIwQYMBaAFNLEsNKR
# 1EwRcbNhyz2h/t2oatTjMBIGA1UdEwEB/wQIMAYBAf8CAQAwMwYIKwYBBQUHAQEE
# JzAlMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5nb2RhZGR5LmNvbTBGBgNVHR8E
# PzA9MDugOaA3hjVodHRwOi8vY2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9z
# aXRvcnkvZ2Ryb290LmNybDBLBgNVHSAERDBCMEAGBFUdIAAwODA2BggrBgEFBQcC
# ARYqaHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNvbS9yZXBvc2l0b3J5MA4G
# A1UdDwEB/wQEAwIBBjANBgkqhkiG9w0BAQUFAAOCAQEA0obA7L35obZn7mYLogY6
# BFCOFXKsSnSVU8s3y0RJ7weQazPZlvCUVqUTMAU8hTIhe8nHCqgkpJDeRtMlIxQD
# Z8IQ1m8PXXt6zJ/FWCrBxJ4hqFrzrKRG857kY8svkKQpKQHZciwp3zcBJ7xP7mjT
# IY/As+T1Ce3SEKpTtL7wzFkL1juWHJUkSd/O7P2nSJEURQ46Nm/aRbNFokHJ1NdE
# Tj65dHbVohNVLMaHo7WZrAaEh391Bvy/FEwOzG7E3z23EnH06PFRQCIoSeAdS4eo
# NMwGot0SWtGGNmQDNW9vd27r8oVQmF6rA1OtkSNjHxaczbmyBWM64fRoGxcFNZVT
# 7jCCBUYwggQuoAMCAQICBwQe+Z3rVhEwDQYJKoZIhvcNAQEFBQAwgcoxCzAJBgNV
# BAYTAlVTMRAwDgYDVQQIEwdBcml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRow
# GAYDVQQKExFHb0RhZGR5LmNvbSwgSW5jLjEzMDEGA1UECxMqaHR0cDovL2NlcnRp
# ZmljYXRlcy5nb2RhZGR5LmNvbS9yZXBvc2l0b3J5MTAwLgYDVQQDEydHbyBEYWRk
# eSBTZWN1cmUgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkxETAPBgNVBAUTCDA3OTY5
# Mjg3MB4XDTE0MDYxMjE5NDEyM1oXDTE2MDYxMjE5NDEyM1owdDELMAkGA1UEBhMC
# VVMxFjAUBgNVBAgTDU1hc3NhY2h1c2V0dHMxDzANBgNVBAcTBkJvc3RvbjEdMBsG
# A1UEChMUUHJveHkgTmV0d29ya3MsIEluYy4xHTAbBgNVBAMTFFByb3h5IE5ldHdv
# cmtzLCBJbmMuMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAtv978VTM
# H2dMJtBo61fkYAGjZRWmOihwCeu/hZf535rH+8GC0OJXZDxYvXzSlFVnvrS/6R1W
# +SK0xL2yqgec3Uvv7R/CftM/dqhqjLMlAv1Q+sf7n11QnkqzeJn1qGcg18pakHuU
# jmPTvY/2SMxMDQ0BMnz1JPoUXbuy7QFxpwX7WGyI2vFRf7NrGqnEny6L2Ze2vWPs
# idigKYVuNZRubdtTkjU9DBuBDJMDyPc7yvpwrL3TSaGWSE8zlG3u/VkA5T+KRD+2
# yJm0aAHs8katpf1XuTCO1cyLBQZlc/+XuYAmMOS+wvodrQpvYWJB0NzlOhhrE7BO
# CfPvHCqTEUOKxwIDAQABo4IBhDCCAYAwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAK
# BggrBgEFBQcDAzAOBgNVHQ8BAf8EBAMCB4AwMwYDVR0fBCwwKjAooCagJIYiaHR0
# cDovL2NybC5nb2RhZGR5LmNvbS9nZHM1LTE2LmNybDBTBgNVHSAETDBKMEgGC2CG
# SAGG/W0BBxcCMDkwNwYIKwYBBQUHAgEWK2h0dHA6Ly9jZXJ0aWZpY2F0ZXMuZ29k
# YWRkeS5jb20vcmVwb3NpdG9yeS8wgYAGCCsGAQUFBwEBBHQwcjAkBggrBgEFBQcw
# AYYYaHR0cDovL29jc3AuZ29kYWRkeS5jb20vMEoGCCsGAQUFBzAChj5odHRwOi8v
# Y2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvZ2RfaW50ZXJtZWRp
# YXRlLmNydDAfBgNVHSMEGDAWgBT9rGEyk2xF1uLuhV+auud2mWjM5zAdBgNVHQ4E
# FgQUqY92Vaco8oUA0CO79l2esDS5tgQwDQYJKoZIhvcNAQEFBQADggEBACMx2cCE
# kirZLB7AtAc3rhVXup7MG3Ogf4kTNnh1VY6hwEtf7c34B4u6R27SB7Sz+qrtA3Ui
# Ll3Q57Xh4oGladL6I6DYVE7XA33QIOt64/mMK749Ke+wZiFW1Js8pXu9v4GS2KXo
# wN9QIu2YGdQL/FH3dWlcLFs3BDbn75ezkF6QDjOnuFaja8ZSqehoakOV+bcRMaCW
# XqdrDM6ZsuXHR5+DeIC6kdTm0jpul3DK2WWUvX77qyewHW84H34skmw3C7GBoAAd
# nIOT8H1/9sut6P7xN2Go+n5FPMgq2b0uISojnxyokYqZ4bxIsn8nbNABDFQ09JrT
# 8RhpKTCfUlGdwyExggSHMIIEgwIBATCB1jCByjELMAkGA1UEBhMCVVMxEDAOBgNV
# BAgTB0FyaXpvbmExEzARBgNVBAcTClNjb3R0c2RhbGUxGjAYBgNVBAoTEUdvRGFk
# ZHkuY29tLCBJbmMuMTMwMQYDVQQLEypodHRwOi8vY2VydGlmaWNhdGVzLmdvZGFk
# ZHkuY29tL3JlcG9zaXRvcnkxMDAuBgNVBAMTJ0dvIERhZGR5IFNlY3VyZSBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eTERMA8GA1UEBRMIMDc5NjkyODcCBwQe+Z3rVhEw
# CQYFKw4DAhoFAKB4MBgGCisGAQQBgjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcN
# AQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUw
# IwYJKoZIhvcNAQkEMRYEFKN/qnMVYXAiYxaxLigRl3FBh10XMA0GCSqGSIb3DQEB
# AQUABIIBAD/ov5oBVXFcBTjjTZFknEw0LR86nc16D2auCYFj7yJxUxeqEkgPUJFP
# m/ZaozwcBnS6o4jyDBZWA4VkNgNOqCCB/HvEN5mdKIu1eZ2EsgJUPYpwooRy+boj
# gy2WOUhVsHQkxAjsvJJjsr2wQZ1f5AEY7WM/MRvRuEQdN0QY6uV7L44/YyQ5GTRH
# rHMSUYgOzPW0OIIws4/VqlfFgTdud1f9o9Gwxc1TT01meQLE4Y5l+xVpdoIhNIrk
# 9dGrI3q3Qmg7NxoY2UB68OHDVeaCqLLA5vZquWP8j0Yc+wlUwRzdIK+SgTEOac5u
# o1NgF1C25I0DTGw5oG/X/GgYCG9ew7yhggILMIICBwYJKoZIhvcNAQkGMYIB+DCC
# AfQCAQEwcjBeMQswCQYDVQQGEwJVUzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9y
# YXRpb24xMDAuBgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMg
# Q0EgLSBHMgIQDs/0OMj+vzVuBNhqmBsaUDAJBgUrDgMCGgUAoF0wGAYJKoZIhvcN
# AQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTYwMjI5MjA0NTEyWjAj
# BgkqhkiG9w0BCQQxFgQUIPKHPlQB5mBlYAM6jPptcjd598YwDQYJKoZIhvcNAQEB
# BQAEggEAjAP3aUaMPjnMi7oiz3OYRt1L4J39kwmFdTmY6JxXMbIsYvqTb8/VwDEG
# F9ZhbGgG6xIEJ9YYenHmmcWKJohfYCEKlvaZ+m706nitGLP26pEBPLMYdQUnMbbS
# 31maTq5EgCUbvPAKxuiOgUeUom4jpBhFYKk2O3B9VCSjhr5mEpolVrpJw17N7RmU
# 3QmAG4sXNT3SZLYM8reaibsCWzHNNIIr/330AQK7Vb9iyhNvC0L+x37e+MmXk4Jj
# 684JNdAhiJSvIrUNYQ24puWVg4QbHc1pJWPtbsr4zrcCK5O9YZAzGhua8wJmoMLh
# 09UsvmCV+bwjaFi29GNFyrgnFLGJ6Q==
# SIG # End signature block
