﻿##############################################################################
##
## Power Shell Module for managing Proxy Objects Security
##
## Exported functions:
##   New-Ace
##   Add-Ace
##   Add-AccessAllowed
##   Find-UserACEs
##   Remove-UserACEs
##
## Exported examples:
##
##
##############################################################################


# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Constants
Import-Module Proxy-Gateway-Utils

#
# Functions
#
<#

.FUNCTION NAME

New-Ace

.SYNOPSIS

Creates a new ACE object "Win32_ACE" with the given User or Group Identity and Access Mask. 

.RELATED

Add-Ace

.EXAMPLE

PS > $ace1 = New-Ace -Domain "ourdomain.com" -Name "alex" -AccessMask 0xABBA

#>
function New-Ace(
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"), 
    [int]$AccessMask = $(throw "Please provide Access Mask"),
    [int]$AceType = $_ACE_Type_Access_Allowed)
{
    # Create an Account Object
    $account = New-Object System.Security.Principal.NTAccount($Domain, $Name)
    # Translate Account Object into Security Identifier
    $secIdent = $account.Translate([System.Security.Principal.SecurityIdentifier])
    # Get binary SID representation
    [byte[]]$binSID = ,0 * $secIdent.BinaryLength
    $secIdent.GetBinaryForm($binSID, 0)
    
    #
    # Setup a new Trustee Object for the given User Name and Domain
    #
    $trustee = ([WMIClass] "Win32_Trustee").CreateInstance()
    $trustee.Domain = $Domain
    $trustee.Name = $Name
    $trustee.SID = $binSID
   
    #
    # Setup a new ACE Object
    #
    $ace = ([WMIClass] "Win32_ACE").CreateInstance()
    $ace.AceFlags = $_ACE_Flag_OBJECT_INHERIT_ACE -bor $_ACE_Flag_CONTAINER_INHERIT_ACE
    $ace.AceType = $AceType
    $ace.AccessMask = $AccessMask
    $ace.Trustee = $trustee
    
    return $ace
}

<#

.FUNCTION NAME

Add-Ace

.SYNOPSIS

Adds new ACE to the binary representation of Security Descriptor.

.RELATED

New-Ace

.EXAMPLE

PS > Import-Module Proxy-Gateway
PS > Connect-Gateway -station 'localhost'
PS > $gw = (Get-Connection).gateway
PS > $gw.addGroup("newGroup", "newGroup description")
PS > 
PS > Import-Module Proxy-Gateway-Utils
PS > $newGroup = Find-Group -proxyGWGroups $gw.getGroups() -name "newGroup"
PS > 
PS > $ace1 = New-Ace -Domain "ourdomain.com" -Name "alex" -AccessMask 0xABBA
PS > $newGroup.security.sd = Add-Ace -BinarySD $newGroup.security.sd -Ace $ace1

#>
function Add-Ace(
    $BinarySD = $(throw "Please provide Security Descriptor as a binary array"), 
    $Ace = $(throw "Please provide ACE (Win32_ACE) Object"))
{
    # Create Security Descriptor(Win32_SecurityDescriptor) class from the binary array
    $helper = [WMIClass]"Win32_SecurityDescriptorHelper"
    $retObj = $helper.BinarySDToWin32SD($BinarySD)
    if($retObj.ReturnValue -ne 0)
    {
        Write-Error -message "Error: BinarySDToWin32SD() returns: $($retObj.ReturnValue)." 
        return $null
    }
    $secDesc = $retObj.Descriptor
    
    # Add ACE to Security Descriptor
    $secDesc.DACL += $Ace.PsObject.BaseObject
    
    # Convert Security descriptor back to binary representation
    $retObj = $helper.Win32SDToBinarySD($secDesc)
    if($retObj.ReturnValue -ne 0)
    {
        Write-Error -message "Error: Win32SDToBinarySD() returns: $($retObj.ReturnValue)." 
        return $null
    }
    
    return $retObj.BinarySD
}

<#

.FUNCTION NAME

Add-AccessAllowed

.SYNOPSIS

Adds new Access Allowed entity to Proxy Security Object with given User or Group Identity and Access Mask.

.RELATED

.EXAMPLE

PS > Import-Module Proxy-Gateway
PS > Connect-Gateway -station 'localhost'
PS > $gw = (Get-Connection).gateway
PS > $gw.addGroup("newGroup", "newGroup description")
PS > 
PS > Import-Module Proxy-Gateway-Utils
PS > $newGroup = Find-Group -proxyGWGroups $gw.getGroups() -name "newGroup"
PS > 
PS > Import-Module Proxy-ObjectSec
PS > Add-AccessAllowed -PrxSecurityObject $newGroup.security -Domain "ourdomain.com" -Name "alex" -AccessMask 0xABBA

#>
function Add-AccessAllowed(
    $PrxSecurityObject = $(throw "Please provide Proxy Security Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"), 
    [int]$AccessMask = $(throw "Please provide Access Mask"))
{
    if( -not ($PrxSecurityObject | Get-Member | where { $_.Name -eq 'sd'}) `
        -or `
        -not ($PrxSecurityObject.sd.GetType().FullName -eq 'System.Byte[]')        
    )
    {
        Write-Error -message "Error: Proxy Security Object doesn't have ''sd'' property." 
        return $false
    }
    
    # Create new ACE for the given User Identity and Access Mask
    $newAce = New-Ace -Domain $Domain -Name $Name -AccessMask $AccessMask
    
    # Add new ACE to Object's Security Descriptor
    $newSD = Add-Ace -BinarySD $PrxSecurityObject.sd -Ace $newAce
    if(-not $newSD)
    {
        Write-Error -message "Error: Can't add new ACE to Security Descriptor." 
        return $false
    }
    
    # Set new Security Descriptor back to the Security Object
    $PrxSecurityObject.sd = [ System.Byte[] ] $newSD
    
    return $true
}

<#

.FUNCTION NAME

Find-UserACEs

.SYNOPSIS

Finds and returns all security settings for the given User Identity in Proxy Security Object as
an array of strings(right names).

.RELATED

.EXAMPLE

PS > Find-UserACEs -PrxSecurityObject $newGroup.security -Domain "ourdomain.com" -Name "alex"

#>
function Find-UserACEs(
    $PrxSecurityObject = $(throw "Please provide Proxy Security Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"), 
    $SecRightsMap = $(throw "Please provide Security Rights Map"))
{
    $retList = New-Object System.Collections.ArrayList
    
    if( -not ($PrxSecurityObject | Get-Member | where { $_.Name -eq 'sd'}) `
        -or `
        -not ($PrxSecurityObject.sd.GetType().FullName -eq 'System.Byte[]')        
    )
    {
        Write-Error -message "Error: Proxy Security Object doesn't have ''sd'' property." 
        return $null
    }
    
    # Create Security Descriptor(Win32_SecurityDescriptor) class from the binary array
    $helper = [WMIClass]"Win32_SecurityDescriptorHelper"
    $retObj = $helper.BinarySDToWin32SD($PrxSecurityObject.sd)
    if($retObj.ReturnValue -ne 0)
    {
        Write-Error -message "Error: BinarySDToWin32SD() returns: $($retObj.ReturnValue)." 
        return $null
    }
    $secDesc = $retObj.Descriptor
    
    # Create an Account Object
    $account = New-Object System.Security.Principal.NTAccount($Domain, $Name)
    # Translate Account Object into Security Identifier
    $secIdent = $account.Translate([System.Security.Principal.SecurityIdentifier])
    
    # Find ACE with given Name and Domain in Security Descriptor
    foreach($ace in $secDesc.DACL)
    {
        # Compare by Domain and User Names
    	#if(($ace.Trustee.Name -ieq $Name) -and ((-not $Domain) -or ($ace.Trustee.Domain -ieq $Domain)))
        
        # Compare by SIDs
        if($ace.Trustee.SIDString -eq $secIdent.ToString())
    	{
    		# Print User ACE's access mask by given $SecRightsMap = @{ accessRightBinaryFlag = "accessRightName" }
            foreach ($key in $SecRightsMap.Keys) 
            { 
                if(($key -band $ace.AccessMask) -eq $key) 
                {
                    $null = $retList.Add([String]($SecRightsMap[$key]))
                } 
            }
    	}
    }
    
    return $retList
}


<#

.FUNCTION NAME

Remove-UserACEs

.SYNOPSIS

Finds and removes all security settings for the given User Identity in Proxy Security Object.

.RELATED

.EXAMPLE

PS > Remove-UserACEs -PrxSecurityObject $newGroup.security -Domain "ourdomain.com" -Name "alex"

#>
function Remove-UserACEs(
    $PrxSecurityObject = $(throw "Please provide Proxy Security Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if( -not ($PrxSecurityObject | Get-Member | where { $_.Name -eq 'sd'}) `
        -or `
        -not ($PrxSecurityObject.sd.GetType().FullName -eq 'System.Byte[]')        
    )
    {
        Write-Error -message "Error: Proxy Security Object doesn't have ''sd'' property." 
        return $false
    }
    
    # Create Security Descriptor(Win32_SecurityDescriptor) class from the binary array
    $helper = [WMIClass]"Win32_SecurityDescriptorHelper"
    $retObj = $helper.BinarySDToWin32SD($PrxSecurityObject.sd)
    if($retObj.ReturnValue -ne 0)
    {
        Write-Error -message "Error: BinarySDToWin32SD() returns: $($retObj.ReturnValue)." 
        return $false
    }
    $secDesc = $retObj.Descriptor
    
    # Create an Account Object
    $account = New-Object System.Security.Principal.NTAccount($Domain, $Name)
    # Translate Account Object into Security Identifier
    $secIdent = $account.Translate([System.Security.Principal.SecurityIdentifier])
    
    # Go through ACEs in DACL and build a new DACL avoiding the ACEs with given identity
    $wasFound = $false
    $newDACL = @()
    foreach($ace in $secDesc.DACL)
    {
        # Compare by SIDs
        if($ace.Trustee.SIDString -eq $secIdent.ToString())
        {
            $wasFound = $true
        }
        else
    	{
            $newDACL += $ace
    	}
    }
    
    # If ACEs with given identity were found, - create new Security Descriptor and copy all
    # properties from the original one except the DACL property which will be assigned from
    # the new one.
    if($wasFound)
    {
        $newDesc = ([WMIClass] "Win32_SecurityDescriptor").CreateInstance()
        $newDesc.ControlFlags =  $secDesc.ControlFlags
        $newDesc.Group = $secDesc.Group
        $newDesc.Owner = $secDesc.Owner
        $newDesc.SACL = $secDesc.SACL
        $newDesc.DACL = [System.Management.ManagementBaseObject[]]$newDACL
        
        # Convert Security descriptor to binary representation
        $retObj = $helper.Win32SDToBinarySD($newDesc)
        if($retObj.ReturnValue -ne 0)
        {
            Write-Error -message "Error: Win32SDToBinarySD() returns: $($retObj.ReturnValue)." 
            return $false
        }

        $PrxSecurityObject.sd = $retObj.BinarySD        
        
        return $true
    }
    
    return $false
}


#
# Module exports
#
Export-ModuleMember -Function  New-Ace, Add-Ace, Add-AccessAllowed, Find-UserACEs, Remove-UserACEs

# SIG # Begin signature block
# MIIXcgYJKoZIhvcNAQcCoIIXYzCCF18CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUAxtnK1doAexiBe6m59FynBh6
# SimgghLuMIIDnzCCAoegAwIBAgIQeaKlhfnRFUIT2bg+9raN7TANBgkqhkiG9w0B
# AQUFADBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xKzAp
# BgNVBAMTIlZlcmlTaWduIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EwHhcNMTIw
# NTAxMDAwMDAwWhcNMTIxMjMxMjM1OTU5WjBiMQswCQYDVQQGEwJVUzEdMBsGA1UE
# ChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xNDAyBgNVBAMTK1N5bWFudGVjIFRpbWUg
# U3RhbXBpbmcgU2VydmljZXMgU2lnbmVyIC0gRzMwgZ8wDQYJKoZIhvcNAQEBBQAD
# gY0AMIGJAoGBAKlZZnTaPYp9etj89YBEe/5HahRVTlBHC+zT7c72OPdPabmx8LZ4
# ggqMdhZn4gKttw2livYD/GbT/AgtzLVzWXuJ3DNuZlpeUje0YtGSWTUUi0WsWbJN
# JKKYlGhCcp86aOJri54iLfSYTprGr7PkoKs8KL8j4ddypPIQU2eud69RAgMBAAGj
# geMwgeAwDAYDVR0TAQH/BAIwADAzBgNVHR8ELDAqMCigJqAkhiJodHRwOi8vY3Js
# LnZlcmlzaWduLmNvbS90c3MtY2EuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMI
# MDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVyaXNp
# Z24uY29tMA4GA1UdDwEB/wQEAwIHgDAeBgNVHREEFzAVpBMwETEPMA0GA1UEAxMG
# VFNBMS0zMB0GA1UdDgQWBBS0t/GJSSZg52Xqc67c0zjNv1eSbzANBgkqhkiG9w0B
# AQUFAAOCAQEAHpiqJ7d4tQi1yXJtt9/ADpimNcSIydL2bfFLGvvV+S2ZAJ7R55uL
# 4T+9OYAMZs0HvFyYVKaUuhDRTour9W9lzGcJooB8UugOA9ZresYFGOzIrEJ8Byyn
# PQhm3ADt/ZQdc/JymJOxEdaP747qrPSWUQzQjd8xUk9er32nSnXmTs4rnykr589d
# nwN+bid7I61iKWavkugszr2cf9zNFzxDwgk/dUXHnuTXYH+XxuSqx2n1/M10rCyw
# SMFQTnBWHrU1046+se2svf4M7IV91buFZkQZXZ+T64K6Y57TfGH/yBvZI1h/MKNm
# oTkmXpLDPMs3Mvr1o43c1bCj6SU2VdeB+jCCA8QwggMtoAMCAQICEEe/GZXfjVJG
# Q/fbbUgNMaQwDQYJKoZIhvcNAQEFBQAwgYsxCzAJBgNVBAYTAlpBMRUwEwYDVQQI
# EwxXZXN0ZXJuIENhcGUxFDASBgNVBAcTC0R1cmJhbnZpbGxlMQ8wDQYDVQQKEwZU
# aGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBDZXJ0aWZpY2F0aW9uMR8wHQYDVQQDExZU
# aGF3dGUgVGltZXN0YW1waW5nIENBMB4XDTAzMTIwNDAwMDAwMFoXDTEzMTIwMzIz
# NTk1OVowUzELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSsw
# KQYDVQQDEyJWZXJpU2lnbiBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBMIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqcqypMzNIK8KfYmsh3XwtE7x38EP
# v2dhvaNkHNq7+cozq4QwiVh+jNtr3TaeD7/R7Hjyd6Z+bzy/k68Numj0bJTKvVIt
# q0g99bbVXV8bAp/6L2sepPejmqYayALhf0xS4w5g7EAcfrkN3j/HtN+HvV96ajEu
# A5mBE6hHIM4xcw1XLc14NDOVEpkSud5oL6rm48KKjCrDiyGHZr2DWFdvdb88qiaH
# XcoQFTyfhOpUwQpuxP7FSt25BxGXInzbPifRHnjsnzHJ8eYiGdvEs0dDmhpfoB6Q
# 5F717nzxfatiAY/1TQve0CJWqJXNroh2ru66DfPkTdmg+2igrhQ7s4fBuwIDAQAB
# o4HbMIHYMDQGCCsGAQUFBwEBBCgwJjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3Au
# dmVyaXNpZ24uY29tMBIGA1UdEwEB/wQIMAYBAf8CAQAwQQYDVR0fBDowODA2oDSg
# MoYwaHR0cDovL2NybC52ZXJpc2lnbi5jb20vVGhhd3RlVGltZXN0YW1waW5nQ0Eu
# Y3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIBBjAkBgNVHREE
# HTAbpBkwFzEVMBMGA1UEAxMMVFNBMjA0OC0xLTUzMA0GCSqGSIb3DQEBBQUAA4GB
# AEpr+epYwkQcMYl5mSuWv4KsAdYcTM2wilhu3wgpo17IypMT5wRSDe9HJy8AOLDk
# yZNOmtQiYhX3PzchT3AxgPGLOIez6OiXAP7PVZZOJNKpJ056rrdhQfMqzufJ2V7d
# uyuFPrWdtdnhV/++tMV+9c8MnvCX/ivTO1IbGzgn9z9KMIIFcTCCBFmgAwIBAgIQ
# XM5xW+LS1ji4A9NViFFAMDANBgkqhkiG9w0BAQUFADCBtDELMAkGA1UEBhMCVVMx
# FzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZWZXJpU2lnbiBUcnVz
# dCBOZXR3b3JrMTswOQYDVQQLEzJUZXJtcyBvZiB1c2UgYXQgaHR0cHM6Ly93d3cu
# dmVyaXNpZ24uY29tL3JwYSAoYykxMDEuMCwGA1UEAxMlVmVyaVNpZ24gQ2xhc3Mg
# MyBDb2RlIFNpZ25pbmcgMjAxMCBDQTAeFw0xMjA2MTUwMDAwMDBaFw0xNDA3MDky
# MzU5NTlaMIG0MQswCQYDVQQGEwJVUzEWMBQGA1UECBMNTWFzc2FjaHVzZXR0czEP
# MA0GA1UEBxMGQm9zdG9uMR0wGwYDVQQKFBRQcm94eSBOZXR3b3JrcywgSW5jLjE+
# MDwGA1UECxM1RGlnaXRhbCBJRCBDbGFzcyAzIC0gTWljcm9zb2Z0IFNvZnR3YXJl
# IFZhbGlkYXRpb24gdjIxHTAbBgNVBAMUFFByb3h5IE5ldHdvcmtzLCBJbmMuMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuORukVR3ArkRbmIr7uMEWljY
# GxNuA2MUK0RKXLO8sswGaEPlSq2KeysSQZVScPGcJtkNj+lIsZ5ECM+bYdAJ4Svv
# BTEqt0HKHsovnv7UKFUCoDOHvVZThWA7+58iFrbMMa/QrPWaqBqYMBGyCtRRZjwf
# pE5kWKe617Kcz0w+myqJv/FmOg7mXe5/j25U21/C48q6uE+CzTvm/fv/DhCOZrMH
# ts641LnzX5I3SNv24pMZ0eUU7hGG9J42u+m3D/yC98DkzWIfYnNCcM2COQCkx2FR
# 6MoB2ny6PN+YpdgF2nD71t2LuJMDxVF6PGbE9b+aNMfraRq1rwPfT8BzPwmavQID
# AQABo4IBezCCAXcwCQYDVR0TBAIwADAOBgNVHQ8BAf8EBAMCB4AwQAYDVR0fBDkw
# NzA1oDOgMYYvaHR0cDovL2NzYzMtMjAxMC1jcmwudmVyaXNpZ24uY29tL0NTQzMt
# MjAxMC5jcmwwRAYDVR0gBD0wOzA5BgtghkgBhvhFAQcXAzAqMCgGCCsGAQUFBwIB
# FhxodHRwczovL3d3dy52ZXJpc2lnbi5jb20vcnBhMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMHEGCCsGAQUFBwEBBGUwYzAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AudmVy
# aXNpZ24uY29tMDsGCCsGAQUFBzAChi9odHRwOi8vY3NjMy0yMDEwLWFpYS52ZXJp
# c2lnbi5jb20vQ1NDMy0yMDEwLmNlcjAfBgNVHSMEGDAWgBTPmanqeyb0S8mOj9fw
# BSbv49KnnTARBglghkgBhvhCAQEEBAMCBBAwFgYKKwYBBAGCNwIBGwQIMAYBAQAB
# Af8wDQYJKoZIhvcNAQEFBQADggEBABDAtzEM2dWEt6wcvi3fSe9Y+hwIS0Oi6Rmf
# QkWMWwHKXvZCOOzuq2i2ywCXnyPz+Fb7waMen0GK2DkfNOwjbjO3yKfDpzv/MHTY
# 1pQjbyf2ySqrRJZ1iqaTDdTe/dhz2PEnp3wIMCM3wX5OCfHEKDUx4BYcLwxaEMAY
# V0j/wJ/vIBdpbDG3GfBUpeRavYjYAEIau7HCMN7lKvWODP09JaePLwfqNzDKB+GY
# cjiEI0gZMY2U4gJ5VygYHGRQslgZ+gVqGg9A6wqWSjqLXBy8C5CRNQmds+e6081Y
# b3WvPbwtI1lGTNmJDCTkJuXDk2S5LDkOEg4DpilD1nN7MiqzRf0wggYKMIIE8qAD
# AgECAhBSAOWqJVb8GobtlsnUSzPHMA0GCSqGSIb3DQEBBQUAMIHKMQswCQYDVQQG
# EwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWdu
# IFRydXN0IE5ldHdvcmsxOjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMu
# IC0gRm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENs
# YXNzIDMgUHVibGljIFByaW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBH
# NTAeFw0xMDAyMDgwMDAwMDBaFw0yMDAyMDcyMzU5NTlaMIG0MQswCQYDVQQGEwJV
# UzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRy
# dXN0IE5ldHdvcmsxOzA5BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3
# dy52ZXJpc2lnbi5jb20vcnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFz
# cyAzIENvZGUgU2lnbmluZyAyMDEwIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
# MIIBCgKCAQEA9SNLXqXXirsy6dRX9+/kxyZ+rRmY/qidfZT2NmsQ13WBMH8EaH/L
# K3UezR0IjN9plKc3o5x7gOCZ4e43TV/OOxTuhtTQ9Sc1vCULOKeMY50Xowilq7D7
# zWpigkzVIdob2fHjhDuKKk+FW5ABT8mndhB/JwN8vq5+fcHd+QW8G0icaefApDw8
# QQA+35blxeSUcdZVAccAJkpAPLWhJqkMp22AjpAle8+/PxzrL5b65Yd3xrVWsno7
# VDBTG99iNP8e0fRakyiF5UwXTn5b/aSTmX/fze+kde/vFfZH5/gZctguNBqmtKdM
# fr27Tww9V/Ew1qY2jtaAdtcZLqXNfjQtiQIDAQABo4IB/jCCAfowEgYDVR0TAQH/
# BAgwBgEB/wIBADBwBgNVHSAEaTBnMGUGC2CGSAGG+EUBBxcDMFYwKAYIKwYBBQUH
# AgEWHGh0dHBzOi8vd3d3LnZlcmlzaWduLmNvbS9jcHMwKgYIKwYBBQUHAgIwHhoc
# aHR0cHM6Ly93d3cudmVyaXNpZ24uY29tL3JwYTAOBgNVHQ8BAf8EBAMCAQYwbQYI
# KwYBBQUHAQwEYTBfoV2gWzBZMFcwVRYJaW1hZ2UvZ2lmMCEwHzAHBgUrDgMCGgQU
# j+XTGoasjY5rw8+AatRIGCx7GS4wJRYjaHR0cDovL2xvZ28udmVyaXNpZ24uY29t
# L3ZzbG9nby5naWYwNAYDVR0fBC0wKzApoCegJYYjaHR0cDovL2NybC52ZXJpc2ln
# bi5jb20vcGNhMy1nNS5jcmwwNAYIKwYBBQUHAQEEKDAmMCQGCCsGAQUFBzABhhho
# dHRwOi8vb2NzcC52ZXJpc2lnbi5jb20wHQYDVR0lBBYwFAYIKwYBBQUHAwIGCCsG
# AQUFBwMDMCgGA1UdEQQhMB+kHTAbMRkwFwYDVQQDExBWZXJpU2lnbk1QS0ktMi04
# MB0GA1UdDgQWBBTPmanqeyb0S8mOj9fwBSbv49KnnTAfBgNVHSMEGDAWgBR/02Wn
# wt3su/AwCfNDOfoCrzMxMzANBgkqhkiG9w0BAQUFAAOCAQEAViLmNKTEYctIuQGt
# VqhkD9mMkcS7zAzlrXqgIn/fRzhKLWzRf3EafOxwqbHwT+QPDFP6FV7+dJhJJIWB
# JhyRFEewTGOMu6E01MZF6A2FJnMD0KmMZG3ccZLmRQVgFVlROfxYFGv+1KTteWsI
# DEFy5zciBgm+I+k/RJoe6WGdzLGQXPw90o2sQj1lNtS0PUAoj5sQzyMmzEsgy5Af
# XYxMNMo82OU31m+lIL006ybZrg3nxZr3obQhkTNvhuhYuyV8dA5Y/nUbYz/OMXyb
# jxuWnsVTdoRbnK2R+qztk7pdyCFTwoJTY68SDVCHERs9VFKWiiycPZIaCJoFLseT
# pUiR0zGCA+4wggPqAgEBMIHJMIG0MQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVy
# aVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdvcmsxOzA5
# BgNVBAsTMlRlcm1zIG9mIHVzZSBhdCBodHRwczovL3d3dy52ZXJpc2lnbi5jb20v
# cnBhIChjKTEwMS4wLAYDVQQDEyVWZXJpU2lnbiBDbGFzcyAzIENvZGUgU2lnbmlu
# ZyAyMDEwIENBAhBcznFb4tLWOLgD01WIUUAwMAkGBSsOAwIaBQCgeDAYBgorBgEE
# AYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwG
# CisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBTfhk5e
# tsG/jUuhrX5Y2wi8NZvZYDANBgkqhkiG9w0BAQEFAASCAQBMp5kA/RwVJEk1UvnB
# x1O73SJudxMyyE+jB4WJQQDpavn+2b2JVjadHPH9IvYtJQpd76gJ8c3zfCesIUsp
# ORYvcV0TXM5kRbqoGBtY7r3zoiWr0hNGDiaT1PdmOCsuUQksvpXZXG6uO4X93Xcq
# YL39GtQUglsMagTnGBMlykIrEjY1oLBGOdMarRspQDdcpCXTmHS9k7JboYIaKXFl
# bLQ8DBZMlFqPSw1Kbv756hgK+p8YeKve2AHwDF91zRI/gh8q7i6X2stNOFG1yQ0C
# 4rkcEcCHJJ/I2mOHkDsOxOvtkV3382xzbvTGNu0ReSXpq0TFf9po3qktiBTgUBjz
# KaHioYIBfzCCAXsGCSqGSIb3DQEJBjGCAWwwggFoAgEBMGcwUzELMAkGA1UEBhMC
# VVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMSswKQYDVQQDEyJWZXJpU2lnbiBU
# aW1lIFN0YW1waW5nIFNlcnZpY2VzIENBAhB5oqWF+dEVQhPZuD72to3tMAkGBSsO
# AwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEP
# Fw0xMjA4MDgyMjU1NTVaMCMGCSqGSIb3DQEJBDEWBBRxaxtfnRPe+iUG9ZRJUEkc
# bR4m7jANBgkqhkiG9w0BAQEFAASBgA46Fp2Nblmjz3fLCv6cwvK1NzkN/exA2vJu
# MLZlKm721rKOZoDnlOj/F2c19Q5nYef5jXiDhVCLFb19gGTAz5RYtDrqoZtRYlEJ
# Gg/i9Dl9npjkvdSQ3mRoW/PYh8Rb50Z5WZfvsrLX6uXSSElvBBESLNY19AEHIZJN
# djCSgTXO
# SIG # End signature block
