############################################################################################
##
## Power Shell Script for adding a new user to the common security settings of 
## Proxy Gateway Server.
##
## To have your own custom security settings for the new users, - change "$AccessMask" flags 
## in Initialize* functions to your own. Constants for the different access rights are in 
## Proxy-ObjectSec module.
##
## Usage:
##
## PS> Import-Module Proxy-Gateway
## PS> Connect-Gateway -station 'example_gateway_station_name'
## PS> $gws = (Get-Connection).gateway
## PS> .\Add-NewGWSUser.ps1 -prxGWS $gws -Domain "example_domain_name" -Name "example_user_name"
##
############################################################################################

param(
	$prxGWS = $(throw "Please provide Gateway Object")
	, [string]$Domain = $(throw "Please provide Domain")
	,  [string]$Name = $(throw "Please provide User or Group Name"))

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-ObjectSec))
{
    Import-Module Proxy-ObjectSec
}
if (-not (Get-Module Proxy-GatewaySec))
{
    Import-Module Proxy-GatewaySec
}
if (-not (Get-Module Proxy-Gateway-Utils))
{
    Import-Module Proxy-Gateway-Utils
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}

<#

.FUNCTION NAME

Initialize-DataServicesSecurity
Initialize-SettingsSecurity
Initialize-OperationSecurity
Initialize-AllHostsGroupHostSecurity

.SYNOPSIS

Group of functions which sets default access rights for the new user.

.RELATED

.EXAMPLE

PS > Import-Module Proxy-Gateway
PS > Connect-Gateway -station 'localhost'
PS > $gws = (Get-Connection).gateway
PS > 
PS > Initialize-DataServicesSecurity -prxGWS $gws -Domain "ourdomain.com -Name "ivan"
PS > Initialize-SettingsSecurity -prxGWS $gws -Domain "ourdomain.com -Name "ivan"
PS > Initialize-OperationSecurity -prxGWS $gws -Domain "ourdomain.com -Name "ivan"
PS > Initialize-AllHostsGroupHostSecurity -prxGWS $gws -Domain "ourdomain.com -Name "ivan"
PS > 

#>

function Initialize-DataServicesSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    $AccessMask = $gatewayConnect -bor $gatewayReadClientConnections -bor `
        $gatewayModifyClientConnections -bor $gatewayReadActiveMasters -bor `
        $gatewayModifyActiveMasters -bor $gatewayReadActiveHosts -bor `
        $gatewayModifyActiveHosts -bor $gatewayReadActiveRecordings -bor `
        $gatewayModifyActiveRecordings -bor $gatewaySendIPCData
    
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.security -Domain $Domain -Name $Name -AccessMask $AccessMask
}

function Initialize-SettingsSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    $AccessMask = $settingsRead -bor $settingsReadLicenses -bor `
        $settingsReadPollSettings -bor $settingsReadProtocols -bor `
        $settingsReadAuditSettings -bor $settingsReadEncryption -bor `
        $settingsReadRecordingSettings
            
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.settings.security -Domain $Domain -Name $Name -AccessMask $AccessMask
}

function Initialize-OperationSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    $AccessMask = $operationRecordToFile -bor $operationConnectToFile -bor `
        $operationImportSession
            
    return Add-AccessAllowed -PrxSecurityObject $prxGWS.operationSecurity -Domain $Domain -Name $Name -AccessMask $AccessMask
}

function Initialize-AllHostsGroupHostSecurity(
    $prxGWS = $(throw "Please provide Gateway Object"), 
    [string]$Domain = $(throw "Please provide Domain"), 
    [string]$Name = $(throw "Please provide User or Group Name"))
{
    if(-not (Test-GatewayObject $prxGWS))
    {
        return $false
    }
    
    $allHostsGroup = $prxGWS.getGroups() | where { $_.GroupType -eq $groupTypeAllHosts }
    if(-not $allHostsGroup)
    {
        Write-Error -message "Can't find 'All Hosts' group." 
        return $false
    }
    
    $AccessMask = $hostRead -bor $hostConnect -bor $hostRemoteView -bor $hostStartRecording
    if(Add-AccessAllowed -PrxSecurityObject $allHostsGroup.hostSecurity -Domain $Domain -Name $Name -AccessMask $AccessMask)
    {
        $AccessMask = $sessionRead -bor $sessionPlay
        return Add-AccessAllowed -PrxSecurityObject $allHostsGroup.sessionSecurity -Domain $Domain -Name $Name -AccessMask $AccessMask
    }
    else
    {
        return $false
    }
}

if(-not (Initialize-DataServicesSecurity -prxGWS $prxGWS -Domain $Domain -Name $Name))
{
	Write-Error -message "Error: Initialize-DataServicesSecurity failed." 
}
elseif(-not (Initialize-SettingsSecurity -prxGWS $prxGWS -Domain $Domain -Name $Name))
{
	Write-Error -message "Error: Initialize-SettingsSecurity failed." 
}
elseif(-not (Initialize-OperationSecurity -prxGWS $prxGWS -Domain $Domain -Name $Name))
{
	Write-Error -message "Error: Initialize-OperationSecurity failed." 
}
elseif(-not (Initialize-AllHostsGroupHostSecurity -prxGWS $prxGWS -Domain $Domain -Name $Name))
{
	Write-Error -message "Error: Initialize-AllHostsGroupHostSecurity failed." 
}
else
{
	Write-Host "Success!"
}
