##############################################################################
##
## Power Shell Script for managing Gateway's Host membership based on AD 
## groups.
## 
## Parameters:
## csv_file - path to csv file with the Host grouping rules.
## gateway - Gateway specifier
## protocol - Gateway connection protocol
##
## Example:
## 
## PS> Set-ExecutionPolicy -Scope Process -executionPolicy RemoteSigned -Force
## PS> Import-Module Proxy-Gateway
## PS> ConfigureAllHostGroupingViaRules.ps1 C:\tmp\hostrules.csv localhost
##
##
## CSV file format:
## 
## Each line of file represents one 'grouping rule' with next values: verb, "distinguished name of OU", depth, gateway group name
## Where:
##  verb - "add" to add the matching Hosts to the specified Gateway group (leaving other memberships alone), 
##          and "move" to move the matching Hosts to the specified Gateway group (removing all other memberships)
##  "distinguished name of OU" - The distinguished name of an OU in the domain (e.g. "OU=TESTOU1_1,OU=TestOU1,DC=development,DC=local").
##          !IMPORTANT! - The distinguished name should be in "" as it has commas in it.
##  depth - An integer value indicating whether this rule applies only to computers that are directly a member of this OU, 
##          or if the rule applies recursively to sub-OUs contained by this OU.  Values are 0=this OU only; N=depth to recurse to; 
##          -1=fully recursive to leaf nodes.
##  Gateway group name - The name of a Gateway-based group that matching Hosts should be placed in.  If this group name does not exist, 
##          the rule fails; this guards against typographical errors in the rule list, and ensures the groups are created by an 
##          administrator with the correct permissions assigned.
##
## First line of CSV file should always be: verb, DN, depth, group
##
## Example CSV file:
## 
## verb, DN, depth, group
## add, "OU=TestOU1,DC=development,DC=local", -1, testgroup1
## add, "OU=TESTOU1_1,OU=TestOU1,DC=development,DC=local", 0, testgroup2
##
## Example Command Line:
##
## PS> .\ConfigureAllHostsOnGateway.ps1 -csv_file '.\ConfigureAllHostGroupingRules.csv' -gateway 'localhost'
## 
##############################################################################
param(
   [string]$csv_file = $(throw "Please provide path to .csv file")
   , [string]$gateway = $(throw "Please provide Gateway station specifier")
   , [string]$protocol ='TCP'
   , [string]$credentials = '')

# Active Directory names
$comp_cat = "Computer"
$ou_cat = "Organizational-Unit"

# Imports
if (-not (Get-Module Proxy-Gateway))
{
    Import-Module Proxy-Gateway
}

# Helper Functions

function Get-CN($dn_string)
{
   $dn = $dn_string.Split(',')
   return ($dn[0] -replace '^cn=')
}

function Is-Category($ad_obj, $category_name)
{
   return (Get-CN $ad_obj.objectCategory.ToString()) -eq $category_name
}

function Find-Computers($DN, $depth)
{
   $computers = @()
   trap 
   {
      Write-Error -message "Exception: $_"
      return ,$computers
   }
   
   try
   {
      $objOU = New-Object System.DirectoryServices.DirectoryEntry("LDAP://$DN")
      $null = $objOU.RefreshCache()
   }
   catch
   {
      Write-Host "Cannot access: $DN" -ForegroundColor Red
      Write-Host "Exception Type: $($_.Exception.GetType().FullName)" -ForegroundColor Red
      Write-Host "Exception Message: $($_.Exception.Message)" -ForegroundColor Red
      $objOU = $null
      return ,$computers
   }

   if ((-not $objOU) -or (-not (Is-Category $objOU $ou_cat)))
   {
      Write-Warning -message "$DN cannot be open or is not an OU"
      return ,$computers
   }

   foreach ($child in ([adsi]$objOU).children) 
   {
      if ((Is-Category $child $comp_cat))
      {
         $computers += $child
      }
      elseif (($depth -ne 0) -and (Is-Category $child $ou_cat))
      {
         # We test only for a '$depth != 0' and substracting '1' from the '$depth' on each 
         # recursive call. In case of '$depth == -1' the recursion will go down to smaller 
         # negative numbers and will stop when there is no more OU childrens in the AD tree.
         $computers += (Find-Computers $child.distinguishedName $($depth - 1))
      }
   }
   
   # ',@()' lets array to be returned as one object and not to 
   # be placed in 'pipeline' as separate items.
   return ,$computers 
}

function Add-HostToGroup($host_machine_name, $group_name, $remove_host_from_other_groups, $gws)
{
   Write-Host "Adding: $host_machine_name -> $group_name"

   # Find group
   $group = $gws.getGroups() | ? { $_.groupName -eq $group_name }
   if(-not $group)
   {
      Write-Warning -Message "Group: $group_name, was not found on the Gateway."
      return $false
   }

   $host_in_Group = $group.getHosts() | ? { $_.machine -eq $host_machine_name }
   if ($host_in_Group)
   {
      Write-Host "Host: <$($host_in_Group.machine)> is already in <$($group.groupName)> group." -ForegroundColor Green
      return $true
   }

   # Find host
   $gws_host = $gws.getHosts() | ? { $_.machine -eq $host_machine_name }
   if (-not $gws_host)
   {
      Write-Warning -Message "Host: $host_machine_name was not found on the Gateway."
      return $false
   }
   
   # Check that we didn't get more than one Host
   if ($gws_host.GetType().FullName -eq "System.Object[]")
   {
      Write-Warning -Message "Host: We found more than one Host with name: $host_machine_name. Resolve ambiguity and run script again."
      foreach($host_tmp in $gws_host)
      {
         Write-Warning -Message "Host name: $($host_tmp.name)"
      }
      return $false
   }

   # Add host to group
   $group.addHost($gws_host)
   Write-Host "Host: <$($gws_host.machine)> added to <$($group.groupName)> group." -ForegroundColor Green

   if ($remove_host_from_other_groups)
   {
      # Remove host from the other Groups
      foreach($host_group in $gws_host.getGroupMemberships())
      {
         if ($host_group.groupName -eq "All Hosts" -or $host_group.groupName -eq $group_name)
         {
            continue;
         }
         else
         {
            $host_group.removeHost($gws_host)
            Write-Host "Host: <$($gws_host.machine)> removed from <$($host_group.groupName)> group." -ForegroundColor Green
         }
      }
   }

   return $true
}

function Invoke-Rule($verb, $DN, $depth, $group, $gws)
{
   try
   {
      [int]$intDepth = [convert]::ToInt32($depth, 10)
   }
   catch
   {
      Write-Host "Cannot convert 'depth':$depth to integer." -ForegroundColor Red
      return $false
   }

   $computers = Find-Computers $DN $intDepth
   if ($computers.Count -gt 0)   
   {
      foreach($comp in $computers)
      {
         Write-Host "Found computer: $($comp.distinguishedName.ToString())"
         $host_machine_name = (Get-CN $comp.distinguishedName.ToString())
         Add-HostToGroup $host_machine_name $group ($verb -eq "move") $gws
      }
   }
   else
   {
      Write-Host "No computers found."
   }
   
   return $true
}

try
{
   $gws = $null # Connection to Gateway Server

   # Read CSV file
   # 
   # Each line of file represents one 'grouping rule' with next values: verb, "distinguished name of OU", depth, gateway group name
   # Where:
   #  verb - add to add the matching Hosts to the specified Gateway group (leaving other memberships alone), 
   #          and move to move the matching Hosts to the specified Gateway group (removing all other memberships)
   #  "distinguished name of OU" - The distinguished name of an OU in the domain (e.g. OU=TESTOU1_1,OU=TestOU1,DC=development,DC=local).
   #          !IMPORTANT! - The distinguished name should be in "" as it has periods in it.
   #  depth - An integer value indicating whether this rule applies only to computers that are directly a member of this OU, 
   #          or if the rule applies recursively to sub-OUs contained by this OU.  Values are 0=this OU only; N=depth to recurse to; 
   #          -1=fully recursive to leaf nodes.
   #  Gateway group name - The name of a Gateway-based group that matching Hosts should be placed in.  If this group name does not exist, 
   #          the rule fails; this guards against typographical errors in the rule list, and ensures the groups are created by an 
   #          administrator with the correct permissions assigned.
   #
   # First line of CSV file should always be: verb, DN, depth, group
   #
   # Example:
   # 
   # verb, DN, depth, group
   # add, "OU=TestOU1,DC=development,DC=local", -1, testgroup1
   # add, "OU=TESTOU1_1,OU=TestOU1,DC=development,DC=local", 0, testgroup2

   $csv = Import-Csv $csv_file
   if (-not $csv)
   {
      Write-Error -message "Could not open $csv"
      return
   }

   # Connect to Proxy Gateway
   Connect-Gateway -station $gateway -protocol $protocol -credentials $credentials
   if(-not (Test-Connection))
   {
      Write-Error -message "Could not connect to Gateway: $gateway : $protocol"
      return
   }
   $gws = Get-GatewayObject

   foreach($rule in $csv)
   {
      Write-Host "Applying rule: $rule"
      if (-not (Invoke-Rule $rule.verb.ToString() $rule.DN.ToString() $rule.depth.ToString() $rule.group.ToString() $gws))
      {
         Write-Error -message "Could not apply rule: $rule"
      }
      Write-Host ""
   }
}
finally
{
   if ($gws) 
   {
      Disconnect-Gateway
      $gws = $null
   }
}

