##############################################################################
##
## Power Shell Script for configuring Host(s) to report to Gateway(s)
##
## This sample script shows how to connect to a Gateway Server, walk the list
## of all managed Hosts, and use remote Host administration to configure them
## to either remove obsolete Gateway configurations, or add new desired
## Gateway configurations, to the Host settings.
##
## Note well that this script MUST be edited to run the "RemoveGatewayFromHost"
## and "AddGatetwayToHost" scripts with the desired parameters to remove & add Gateways.
##
## Authentication Issues:
##
## A user who has rights to enumerate all (or even a subset) of the Hosts on a Gateway
## can use this script, but may not have access rights to reconfigure the Host settings.
## To work around this, you have a couple of choices:
## (1) Run the script as a member of "Domain Admins", or use the -credentials parameter
##     to specify the a domain administrator account and password (in the form "domain\account|password").
##     This works well if all machines belong to one domain.
## (2) Use the "RemoteControlGateway" account for authentication to the Host.  This
##     requires using another account with Gateway rights to connect to the Gateway, and
##     the RemoteControlGateway account for end-to-end authentication to the Host.
##     This works well if the machines belong to multiple domains, but there are trust
##     relationships among them, and the "RemoteControlGateway" account (with that exact name)
##     exists in one of the domains.  To specify different credential to the Host than the
##     Gateway, use the -credentials form "GWDOMAIN\account4gateway|password`nHOSTDOMAIN\account4host|password".
##     The "`n" syntax in PowerShell inserts a linefeed character, which delimits the two sets of credentials.
##     In the domain "MYDOMAIN", you can use the administrator account and RemoteControlGateway account
##     with the format: "MYDOMAIN\administrator|adminpassword`nMYDOMAIN\RemoteControlGateway|rcgpassword".
##
##
## Examples:
## 
## PS> .\ConfigureAllHostsOnGateway.ps1 -gateway 'localhost'
## 
##############################################################################

param(
	[string]$gateway = $(throw "Please provide Gateway Identifier")
    , [string]$protocol ='TCP'
    , [string]$credentials = '')

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-HostAdmin))
{
    Import-Module Proxy-HostAdmin
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}
if (-not (Get-Module Proxy-Gateway))
{
    Import-Module Proxy-Gateway
}

# Make sure there are no current connections

disconnect-host
disconnect-gateway

# Connect to Gateway and get list of All Hosts

$result = Connect-Gateway -station $gateway -protocol $protocol -credentials $credentials
$gw = Get-GatewayObject
if (-not $gw)
{
    Write-Error -message "Failed to connect to Gateway Server - exiting"
    return
}

# For each host in the "All Hosts" group, connect Host Administration object & process it

foreach ($hh in $gw.getWorkstations())
{
	if (($hh.isManaged -eq 0) -or ($hh.protocol -eq 'internal') -or ($hh.address -eq ''))
	{
		continue
	}
	$msg = "Processing Host on " + $hh.machine + " (HostID=" + $hh.key + ")"
	Write-Output $msg
	$hostHA = Connect-Host -gateway $gateway -hostkey $hh.key -protocol $protocol -credentials $credentials
	if(-not $hostHA)
	{
		# Note: error reported by Connect-Host cmdlet
		# $msg = "Cannot connect to Host with key = " + $hh.key
		# Write-Error -message $msg
		continue
	}

### HANDLE EXCEPTIONS HERE - if some machines shouldn't be touched, list them here by machine name

	if (($hh.machine -eq 'FILESERVER'))
	{
		Disconnect-Host
		continue
	}

### === Fill in desired configuration changes here!
#
# In a sample test environment, the Gateways are:
#     GWS ........... 'w={0AD53915-CDC6-466E-ABEB-DB088ACF8299}'
#     GWC2012 ....... 'w={B032A93D-044B-4CA6-848D-D5D5E7D8CE99}'
#     GATEWAY ....... 'w={1CED4B14-3018-44C7-9ADD-FCC4DDE6A599}'
# In this example, the first two gateways are being retired, and are replaced by the third one

# First, remove the configurations that may already be set that are obsolete/not desired

	.\RemoveGatewayFromHost.ps1 -gwid 'w={0AD53915-CDC6-466E-ABEB-DB088ACF8299}' -gatewayList @('gws', 'gws.mydomain.local')
	.\RemoveGatewayFromHost.ps1 -gwid 'w={B032A93D-044B-4CA6-848D-D5D5E7D8CE99}' -gatewayList @('gwc2012', 'gwc2012.mydomain.local')

# Finally, add in the configurations that are desired

	.\AddGatewayToHost.ps1 -gwid 'w={1CED4B14-3018-44C7-9ADD-FCC4DDE6A599}' -gateway 'gateway.mydomain.local' -protocol 'TCP'

	# Disconnect Host Administration object
	Disconnect-Host
}

