﻿##############################################################################
##
## Power Shell Script for exploring Host Security settings.
##
##
## Examples:
## 
## PS > $hostHA = Connect-Host -gateway "hostname.domain.com" -hostkey "w={07C9B233-1BA8-4A48-81C8-2454E537C479}" -protocol "TCP" -credentials "domain\user|password"
## PS> .\Get-HostSecurity.ps1 $hostHA
## PS> Disconnect-Host
## 
##############################################################################
param($prxHA = $(throw "Please provide Proxy Security Object"))

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}


if(-not $prxHA)
{
    Write-Error -message "Error: First parameter is not Host Administration Object." 
    return 
}

function Write-HostSecurity($PrxSecurityObject, $SecRightsMap)
{
    if( -not ($PrxSecurityObject | Get-Member | where { $_.Name -eq 'sd'}) `
        -or `
        -not ($PrxSecurityObject.sd.GetType().FullName -eq 'System.Byte[]'))
    {
        Write-Error -message "Error: Proxy Security Object doesn't have ''sd'' property." 
        return
    }

    # Create Security Descriptor(Win32_SecurityDescriptor) class from the binary array
    $helper = [WMIClass]"Win32_SecurityDescriptorHelper"
    $retObj = $helper.BinarySDToWin32SD($PrxSecurityObject.sd)
    if($retObj.ReturnValue -ne 0)
    {
        Write-Error -message "Error: BinarySDToWin32SD() returns: $($retObj.ReturnValue)." 
        return $null
    }
    $secDesc = $retObj.Descriptor

    # Find ACE with given Name and Domain in Security Descriptor
    foreach($ace in $secDesc.DACL)
    {
        # Output Domain and User Names
		
		if($ace.Trustee.Name)
		{
			$userAccount = "$($ace.Trustee.Domain) : $($ace.Trustee.Name)"
		}
		else
		{
			try 
			{
				$objSID = New-Object System.Security.Principal.SecurityIdentifier($ace.Trustee.SIDString)
				$objUser = $objSID.Translate([System.Security.Principal.NTAccount])
				$userAccount = $objUser.Value
			} 
			catch
			{
				$userAccount = $ace.Trustee.SIDString
			}
		}
		
		Write-Host 
		Write-Host "User: $userAccount"
        Write-Host "Rights: "
		
        # Output SIDs
		# Print User ACE's access mask by given $SecRightsMap = @{ accessRightBinaryFlag = "accessRightName" }
        foreach ($key in $SecRightsMap.Keys) 
        { 
            if(($key -band $ace.AccessMask) -eq $key) 
            {
                Write-Host "$($SecRightsMap[$key])"
            } 
        }
    }
}

Write-Host "Admin Security: "
Write-HostSecurity $prxHA.settings.adminSecurity $hostAdminSecRightsMap

Write-Host 
Write-Host "Service Security: "
Write-HostSecurity $prxHA.settings.serviceSecurity $hostServiceSecRightsMap

Write-Host 
Write-Host "Settings Security: "
Write-HostSecurity $prxHA.settings.settingsSecurity $hostSettingsSecRightsMap


