##############################################################################
##
## Power Shell Script for reporting on basic information about a Gateway Server
##
## Examples:
## 
## PS> .\ReportGatewayInfo.ps1 -gateway 'localhost'
## 
##############################################################################

param(
	[string]$gateway = $(throw "Please provide Gateway Identifier")
    , [string]$protocol ='TCP'
    , [string]$credentials = '')

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}
if (-not (Get-Module Proxy-Gateway))
{
    Import-Module Proxy-Gateway
}

# Make sure there are no current connections

disconnect-gateway

# Connect to Gateway

$result = Connect-Gateway -station $gateway -protocol $protocol -credentials $credentials
$gw = Get-GatewayObject
if (-not $gw)
{
    Write-Error -message "Failed to connect to Gateway Server - exiting"
    return
}

# Walk a few Gateway collections, and collect the following information:
#
# noManagedWorkstations = number of managed workstations
# noManagedWorkstationsKnownGateway = number where this GWS is a "known Gateway"
# noUnmanagedWorkstations = number of unmanaged workstations
# noHostControlConnections = number of active Host control connections
# noHostPendingStatusConnections = number of active Host status connections in queue

$noManagedWorkstations = 0
$noManagedWorkstationsKnownGateway = 0
$noUnmanagedWorkstations = 0
$noHostControlConnections = 0
$noHostPendingStatusConnections = 0

# For each host in the "All Hosts" group, categorize it

foreach ($hh in $gw.getWorkstations())
{
	if ($hh.protocol -eq 'internal')
	{
		continue
	}
	if ($hh.isManaged -ne 0)
	{
		$noManagedWorkstations = $noManagedWorkstations + 1
		if ($hh.isKnownGateway -ne 0)
		{
			$noManagedWorkstationsKnownGateway = $noManagedWorkstationsKnownGateway + 1
		}
	}
}

$noUnmanagedWorkstations = $gw.settings().getUnmanagedWorkstations().Count()

Write-Output "Number of Managed Workstations        = $noManagedWorkstations"
Write-Output " of these, # configured for this GWS  = $noManagedWorkstationsKnownGateway"
Write-Output "Number of Unmanaged Workstations      = $noUnmanagedWorkstations"

foreach ($hh in $gw.getActiveHostCtrlConnections())
{
	if ($hh.isConnected -eq 0)
	{
		continue
	}
	if ($hh.isControlConnection -eq 0)
	{
		$noHostPendingStatusConnections = $noHostPendingStatusConnections + 1
	}
	else
	{
		$noHostControlConnections = $noHostControlConnections + 1
	}
}

Write-Output "Number of active Host Control Conns   = $noHostControlConnections"
Write-Output "Number of active pending status Hosts = $noHostPendingStatusConnections"

disconnect-gateway
