##############################################################################
##
## Power Shell Script for managing Host Settings
##
## Example function to manipulate Host Settings. 
## Sets 'default' Host parameters according to 'PHSETUP syntax examples' 
## in 'Proxy Pro Host Help'
##
## Examples:
## 
## PS> .\Set-DefaultHostSettings.ps1 -gateway 'localhost' -hostkey '07C9B233-1BA8-4A48-81C8-2454E537C479' # Using logged-in user's credentials
## PS> .\Set-DefaultHostSettings.ps1 -gateway 'localhost' -hostkey '07C9B233-1BA8-4A48-81C8-2454E537C479' -credentials "domain\name|password"
## 
## PS> Set-ExecutionPolicy -Scope Process -executionPolicy RemoteSigned -Force
## PS> Import-Module Proxy-Gateway
## PS> Connect-Gateway 'localhost'
## PS> $gws.getHosts() | where { $_.machine -ne 'virtual' } | % { .\Set-DefaultHostSettings.ps1 'localhost' $_.workstationKey }
## PS> Disconnect-Gateway
## 
##############################################################################

param(
	[string]$gateway = $(throw "Please provide Gateway Identifier")
	, [string]$hostkey = $(throw "Please provide Host Key")
    , [string]$protocol ='TCP'
    , [string]$credentials = '')

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-HostAdmin))
{
    Import-Module Proxy-HostAdmin
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}

# Connect Host Administration object
$hostHA = Connect-Host -gateway $gateway -hostkey $hostkey -protocol $protocol -credentials $credentials
if(-not $hostHA)
{
	Write-Error -message "Can't connect to Host." 
    return $false
}

# This property gets or sets the station name of the Proxy Host.
# $hostHA.settings.stationName = "`%NAME`%"

# Controls whether a beep (or custom sound) is played when a connection is made or terminated.
$hostHA.settings.beepOnConnect = $true

# Enables or disables a periodic beep (or custom sound) while a connection is in progress.
$hostHA.settings.beepEvery = $false

# Sets or gets a value which controls whether the Host tray icon is displayed when there is no connection.
$hostHA.settings.idleAppearance = $windowIcon

# Sets or gets a value which controls whether the Host tray icon is displayed when a connection is active.
$hostHA.settings.connectedAppearance = $windowIcon

# Indicates which Host notifications will be fired.
$hostHA.settings.hostNotifications = $notifyOnConnectDisconnect

# Enables or disables the use of Windows security (authentication).
$hostHA.settings.useWindowsSecurity = $true

# Controls whether remote administration of the Host is allowed
$hostHA.settings.allowRemoteAdmin = $true

# Enables or disables the lock workstation on disconnect feature. If this property is set to True, the workstation is locked when a remote control session ends.
$hostHA.settings.lockWorkstationOnDisconnect = $false

# Enables or disables the reboot on disconnect feature. If this property is set to True, the Host computer will reboot when a remote control connection is terminated
$hostHA.settings.reboot = $false

# Sets or gets a value that controls whether local keyboard and mouse input is suppressed.
$hostHA.settings.keySuppression = $suppressPermit

# Sets or gets the Host access setting. The value of this property will be one of the values in the HostAccess enum.
$hostHA.settings.access  = $accessPermit

# Sets or gets the Host permission setting. The value of this setting determines what type of permission is required to access the Host.
$hostHA.settings.permission = $permitNone

# Sets or gets the number of seconds that are allowed to elapse while waiting for permission to connect to the Host. 
# If the Host does not grant permission within this amount of time, permission is assumed to be denied, and the connection is terminated.
$hostHA.settings.grantTime = 30 

# Sets or gets the number of seconds that are allowed to elapse while requesting permission to connect to the Host. 
# If the Host does not respond within this amount of time, permission is assumed to be granted, and the connection continues.
$hostHA.settings.requestTime = 30

# Controls whether the Host console is automatically locked at connect time, if Connection Permission is requested from the Host, 
# and the user at the console does not respond to the dialog prompt.
$hostHA.settings.lockWorkstationOnDefault = $false

# Sets or gets the manageVisualEffects setting, which controls the how visual effects are managed on the Host.
$hostHA.settings.manageVisualEffects = $managePermit

# Sets or gets the list of Windows visual effects to be disabled when a connection is established. 
# The value of this property can be any of the values in the VisualEffectFlags enum, or an OR of multiple VisualEffectFlags values.
$hostHA.settings.visualEffects = $veAllEffects

# Allow to use UDP protocol for connections to this Host
$protocolIP  = $hostHA.settings.getProtocols() | ? { $_.protocolName -eq "IP" }
if($protocolIP) { $protocolIP.isEnabled = $true }

# Allow to use TCP protocol for connections to this Host
$protocolTCP  = $hostHA.settings.getProtocols() | ? { $_.protocolName -eq "TCP" }
if($protocolTCP) { $protocolTCP.isEnabled = $true }

# Enables or disables the encryption of Host connections. This property must be set before a new connection is established.
$hostHA.settings.useEncryption = $true

# Set preffered ciphers to use with connection encription
$hostHA.settings.preferredCiphers = "A4A3A233R2"

# Determines whether remote control connections to this Host are required to be from Gateways in the list of configured Gateways. 
# If this property is set to False, this Host can accept both Gateway and peer-to-peer connections.
$hostHA.settings.connectByGateway = $false

# Remove all existing Gateway Configs
# $hostHA.settings.getGatewayConfigs() | % { $_.deleteGatewayConfig() }

# Controls whether the Host prefers User Mode Screen Capture as the screen capture technology, or prefers Kernel-mode screen capture.
$hostHA.settings.preferUserMode = $true

# Set user-mode profile
$profile  = $hostHA.settings.getUserModeProfiles() | ? { $_.description -eq "High Quality/High Bandwidth" }
if($profile) { $hostHA.settings.currentUserModeProfile = $profile }


# Disconnect Host Administration object
Disconnect-Host

return $true