﻿############################################################################################
##
## Power Shell Script for setting new owner of ProxySecurity Object.
##
## Usage:
##
## PS> Import-Module Proxy-HostAdmin
## PS> $hostHA = Connect-Host -gateway "localhost" -hostkey "w={09AE457F-FF3A-4ADA-BC78-E808293E80C8}"
## PS> .\Set-Owner.ps1 -SecurityObject $hostHA.settings.adminSecurity -Domain "example_domain_name" -Name "example_user_name"
## PS> Disconnect-Host
##
############################################################################################

param(
    $SecurityObject = $(throw "Please provide Proxy Security Object")
	, [string]$Domain = $(throw "Please provide Domain")
	, [string]$Name = $(throw "Please provide User or Group Name"))

# Check if we got right object
if( -not ($SecurityObject | Get-Member | where { $_.Name -eq 'sd'}) `
    -or `
    -not ($SecurityObject.sd.GetType().FullName -eq 'System.Byte[]')        
)
{
    Write-Error -message "Error: Proxy Security Object doesn't have ''sd'' property." 
    return $false
}

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-ObjectSec))
{
    Import-Module Proxy-ObjectSec
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}

#
# Convert binary Security Descriptor blob to "Win32_SecurityDescriptor" object
#
$helper = [WMIClass]"Win32_SecurityDescriptorHelper"
$secObj = $helper.BinarySDToWin32SD($SecurityObject.sd)
if($secObj.ReturnValue -ne 0)
{
    Write-Error -message "Error: BinarySDToWin32SD() returns: $($secObj.ReturnValue)." 
    return $false
}

#
# Setup a new Trustee Object for the given User Name and Domain
#

# Create an Account Object
$account = New-Object System.Security.Principal.NTAccount($Domain, $Name)
# Translate Account Object into Security Identifier
$secIdent = $account.Translate([System.Security.Principal.SecurityIdentifier])
# Get binary SID representation
[byte[]]$binSID = ,0 * $secIdent.BinaryLength
$secIdent.GetBinaryForm($binSID, 0)

$trustee = ([WMIClass] "Win32_Trustee").CreateInstance()
$trustee.Domain = $Domain
$trustee.Name = $Name
$trustee.SID = $binSID

#
# Build new Security Descriptor Object as a copy of old one with a new Owner
#
$newDesc = ([WMIClass] "Win32_SecurityDescriptor").CreateInstance()
$newDesc.ControlFlags = $secObj.Descriptor.ControlFlags
$newDesc.Group = $secObj.Descriptor.Group
$newDesc.DACL = $secObj.Descriptor.DACL
$newDesc.SACL = $secObj.Descriptor.SACL
$newDesc.Owner = $trustee

#
# Convert new Security Descriptor Object to binary blob and set it to original .sd property of $SecurityObject
#
$newDescBlob = $helper.Win32SDToBinarySD($newDesc)
$SecurityObject.sd = $newDescBlob.BinarySD

return $true